      subroutine calvdf(deltat)
c
c     this subroutine prepares variables for vertical diffusion 
c     calculations on the coarse grid
c
c$Log: calvdf_o.f,v $
c  Revision 1.4  1999/12/21  19:27:27  tcm
c  Renamed com3 include file to comd3.
c
c  Revision 1.3  1999/07/07  16:07:40  tcm
c  Added tracking of fluxes for process analysis.  Some restructuring
c  of code was necessary to accomplish this.
c
c  Revision 1.2  1994/02/10  20:50:56  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c
c
c
c     pertinent information used are:
c     conc( , , , , ) - concentration array (ppm) 
c     h( , , )        - array of heights (km agl)
c     kv( , , )       - array of vertical diffusivities (cm**2/s)
c     depvel( , , )   - array of deposition velocities (m/s)
c     deltat          - time step (min)
c
c....modified 7/22/91 to fix time step calculation -- rrs
c
c....modified 2/11/92, tcm:  added calculation of max time step
c    for deposition calculation
c
c....modified 4/3/92, tcm:  make inner loop the loop over x
c    in order to lengthen vectorizable loops.  move vrtdif code
c    into this routine.  restructure ifs to eliminate branches
c    within inner loops.
c
c....modified 4/1/93, cae: added minimum kv = 1. in denominator
c    of time step calculation so we don't get divide by 0 when kv = 0.
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'balanc.cmd'
      include 'comuam.cmd'
      include 'comd3.cmd'
c
      dimension cflux(mxx,mxz),dz(mxx,mxz),hh(mxx,mxz)
      dimension dzpr(mxx,mxz), ctm(mxx,mxz), dep(mxx)
c
czswa---------------------------------------------------------------
c
c  add two dimensions to tally the fluxes on the boundaries
c
      include 'irmb.cmd'
      dimension flux1(mxx,mxz), flux2(mxx,mxz)
c
czswz---------------------------------------------------------------
c
c
c     fact1 - converts from km**2 to cm **2
c     fact2 - converts from sec to min
c
      data fact1/1.e10/, fact2/0.0166667/
c
      fftmp = 0.15*fact1*fact2
c
c     loop on horizontal grid
c
      do 500 j=1,noy
c
c     see if in our modeling domain
c
        if (istr(j).gt.nox) go to 500
        i1=istr(j) + 1
        i2=iend(j) - 1
        dtv=1.e20
c
cdir$ shortloop
        do 110 i=i1,i2
c--------------------------------------------------------------
c     calculate time step for vertical diffusion based on the
c     stability criterion
c-------------------------------------------------------------
          dzpr(i,1) = h(i,j,1)
          hh(i,1)=h(i,j,1)/2.
  110   continue
        do 120 k=2,nlayer
cdir$ shortloop
          do 115 i=i1,i2
            hh(i,k)=h(i,j,k-1)+0.5*(h(i,j,k)-h(i,j,k-1))
            dz(i,k-1)=hh(i,k)-hh(i,k-1)
            dzpr(i,k) = h(i,j,k) - h(i,j,k-1)
  115     continue
  120   continue
c
cdir$ shortloop
        do 125 i=i1,i2
          dttstu=fftmp*dz(i,1)*dzpr(i,1)/(kv(i,j,1)+1.)
          dtv = amin1(dtv,dttstu)
  125   continue
        do 140 k=3,nlayer-1
cdir$ shortloop
          do 130 i=i1,i2
            dttstu=fftmp*dz(i,k-1)*dzpr(i,k-1)/(kv(i,j,k-1)+1.)
            dttstl=fftmp*dz(i,k-2)*dzpr(i,k-1)/(kv(i,j,k-2)+1.)
            dtv=amin1(dtv,dttstl,dttstu)
  130     continue
  140   continue
cdir$ shortloop
        do 150 i=i1,i2
          dttstl=fftmp*dz(i,nlayer-1)*dzpr(i,nlayer)/
     &           (kv(i,j,nlayer-1)+1.)
          dtv=amin1(dtv,dttstl)
  150   continue
c
c     calculate layer interface diffusive flux coefficient
c
        do 330 k=1,nlayer-1
cdir$ shortloop
          do 320 i=i1,i2
            cflux(i,k)=kv(i,j,k)/(dz(i,k)*fact1*fact2)
  320     continue
  330   continue
c
c     loop on species
c
        do 460 l=1,nospec
c
czswa---------------------------------------------------------------
c  initialize the fluxes to zero
c
           do k = 1, mxz
           do i = 1, mxx
              flux1(i,k) = 0.0
              flux2(i,k) = 0.0
           enddo
           enddo
c
czswz---------------------------------------------------------------
c
c      load deposition rate
c      calculate step size for deposition (make it one quarter
c      of max possible)
c
          dtnew = dtv
cdir$ shortloop
          do 340 i=i1,i2
            dep(i) = 0.06*depvel(i,j,l)
            deldep = -dzpr(i,1)/amin1(dep(i),-1.e-6) * .25
            dtnew = amin1(deldep,dtnew)
  340     continue
          nstep=int(deltat/dtnew)+1
          deltdf=deltat/float(nstep)
c
c   start of vrtdif
c
         do 420 istep=1,nstep
c
c   do it for the first level (vertical diffusion and deposition)
c
           do 350 i=i1,i2
              delc1 = deltdf*dep(i)*conc(i,j,l,1)/dzpr(i,1)
c          dpflow in moles, delc1 in micro-moles/m**3;
c          deltax, deltay, and h in km
              dpflow(l) = dpflow(l) + delc1*deltay*deltax(j)
     &               *h(i,j,1)*1000.
              v1 = deltdf*(dep(i)*conc(i,j,l,1))/dzpr(i,1)
              v2 = deltdf*cflux(i,1)*(conc(i,j,l,2)-conc(i,j,l,1))
     &             /dzpr(i,1)
              ctm(i,1) = conc(i,j,l,1) + v1 + v2
c
              flux1(i,1) = flux1(i,1) + v1
              flux2(i,1) = flux2(i,1) + v2
c
  350      continue
c
c    continue it for levels 2 to layer-next-to-the-top
c
           do 370 k=2,nlayer-1
c
             do 360 i=i1,i2
               v1 = deltdf*cflux(i,k-1)*
     &                (conc(i,j,l,k)-conc(i,j,l,k-1))/dzpr(i,k)
               v2 = deltdf*cflux(i,k)*
     &                (conc(i,j,l,k+1)-conc(i,j,l,k))/dzpr(i,k)
               ctm(i,k) = conc(i,j,l,k) + v2 - v1
c
              flux1(i,k) = flux1(i,k) - v1
              flux2(i,k) = flux2(i,k) + v2
c
  360        continue
  370      continue
c
c    do the final layer
c
           do 380 i=i1,i2
             v1 = deltdf*cflux(i,nlayer-1)*(conc(i,j,l,nlayer)
     &               - conc(i,j,l,nlayer-1))/dzpr(i,nlayer)
             v2 = 0.0
             ctm(i,nlayer) = conc(i,j,l,nlayer) + v2 - v1
c
              flux1(i,nlayer) = flux1(i,nlayer) - v1
              flux2(i,nlayer) = flux2(i,nlayer) + v2
c
  380      continue
c
           do 400 k=1,nlayer
             do 390 i=i1,i2
               conc(i,j,l,k) = ctm(i,k)
  390        continue
  400      continue
c
c   go to the next time step
c
  420    continue
c
c   end of vrtdif
c
c  tally the change in conc due to diffusion at the top/bottom
c  boundaries for ipram
c
           do 402 k=1,nlayer
             do 401 i=i1,i2
               iflg = iflag(i,j)
               if (iflg.gt.0) then
                  btflux(1,iflg,l,k) = flux1(i,k) +
     &                   btflux(1,iflg,l,k)
                  btflux(2,iflg,l,k) = flux2(i,k) +
     &                   btflux(2,iflg,l,k)
               endif
  401        continue
  402      continue
c
czswz-----------------------------------------------------------------
c
460     continue
500   continue
      return
      end 
