      subroutine diffsf(nxf,nyf,nzf,delfx,delfy,uf,vf,fkh)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: diffsf.f,v $
c  Revision 1.3  1996/06/14  22:56:37  tcm
c  modified to use dx*dy rather than dy**2, by gem
c
c  Revision 1.2  1994/02/10  20:54:44  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c
c  Rep[laced deltay**2 with deltay*deltax(j)
c
c
c
c     routine calculates horizontal diffusion coeficients for fine 
c     grids based on scale considerations and deformation of the
c     windfield
c
c     EMPIRICAL PARAMETER DIFCOF, BASED ON SCALE CONSIDERATIONS
c     -- changed to adjust for scale internally.  Default value
c        for DIFCOF is 0.25/SQRT(2).  It is multiplied by 
c        (delfy**2) within this subroutine. (tcm, 11/21/90)
c
c     -- changed to deal with optional staggered u/v grids
c        (Some comments regarding this change:  For grids with 
c        u & v at cell centers, gradients wrt x are calculated
c        at i + 1/2 and gradients wrt y are calculated at j + 1/2.
c        KH is used as if at i + 1/2 in STEPX, but as if at
c        j + 1/2 in STEPY.  On the other hand, in the case of
c        staggered u & v grids, dudx and dvdy are calculated at
c        cell centers and dudy and dvdx are calculated at
c        (i + 1/2, j + 1/2).  KH will still be used as if at
c        i + 1/2 in STEPX and j + 1/2 in STEPY.  In the staggered
c        grid, there is not sufficient information to calculate
c        all the gradients for i = 1 or j = 1.  Rather than 
c        calculate DEF based on incomplete data, DEF is set
c        to zero for these cases.  So that all boundaries are 
c        treated equivalently in either the staggered or cell 
c        centered case, DEF is set to zero for i = 1 or nx
c        or j = 1 or ny.
c        (tcm, 7/15/91)
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'comuam.cmd'
      include 'advec.cmd'
      dimension uf(nxf,nyf,nzf), vf(nxf,nyf,nzf), fkh(nxf,nyf,nzf)
      dimension delfx(nyf)
c
      do 150 k=1,nzf
        do 100 j=1,nyf
          do 90 i=1,nxf
            if (i .eq. 1 .or.
     &          j .eq. 1 .or.
     &          i .eq. nxf .or.
     &          j .eq. nyf) then
              def = 0.
            else
              if (lstagr) then
C *** CALCULATE D(WX)/DX AND D(WY)/DX
                u2=uf(i,j,k)
                u1=uf(i-1,j,k)
                v2=vf(i+1,j,k)
                v1=vf(i,j,k)
                dudx= (u2-u1)/delfx(j)
                dvdx= (v2-v1)/delfx(j)
C *** CALCULATE D(U)/DY AND D(V)/DY
                u2=uf(i,j+1,k)
                u1=uf(i,j,k)
                v2=vf(i,j,k)
                v1=vf(i,j-1,k)
                dudy=(u2-u1)/delfy
                dvdy=(v2-v1)/delfy
C
              else
C *** CALCULATE D(WX)/DX AND D(WY)/DX
                u2=uf(i+1,j,k)
                u1=uf(i,j,k)
                v2=vf(i+1,j,k)
                v1=vf(i,j,k)
                dudx= (u2-u1)/delfx(j)
                dvdx= (v2-v1)/delfx(j)
C *** CALCULATE D(U)/DY AND D(V)/DY
                u2=uf(i,j+1,k)
                u1=uf(i,j,k)
                v2=vf(i,j+1,k)
                v1=vf(i,j,k)
                dudy=(u2-u1)/delfy
                dvdy=(v2-v1)/delfy
              endif
              def=sqrt((dudy+dvdx)**2 + (dudx-dvdy)**2)
            endif
C
C *** CALCULATE KH
C
C
            fkh(i,j,k)=difcof*(delfy*delfx(j))*def
            if (fkh(i,j,k).gt.difmax) fkh(i,j,k)=difmax
            if (fkh(i,j,k).lt.difmin) fkh(i,j,k)=difmin
   90     continue
  100   continue
  150 continue
      return
      end
