      subroutine dochem(time,nyr,nmth,nday,nospec,ncells,
     &      ig,jg,kg,dt,tcell,pcell,zavght,lpreac,lerdch,nsrc)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: dochem.f,v $
c  Revision 1.13  1999/12/21  21:08:48  tcm
c  Cleaned up some format statements.
c
c  Revision 1.11  1999/07/07  16:17:02  tcm
c  Modified step4 calls for consistency with the pa/irr modifications.
c
c  Revision 1.10  1999/05/18  18:52:56  dfg
c  Added passing local radical rcon to step4.
c
c  Revision 1.9  1999/05/18  18:21:08  dfg
c  Pass dt to Step4 instead of dthr because unit in chem routines are per minute.
c
c  Revision 1.8  1999/05/17  22:23:45  dfg
c  Added one local variable for radical and set radical concentrations to 1.0e-9 before call step4.
c
c  Revision 1.7  1999/04/08  23:57:10  tcm
c  Added branch based on input flag to use either fast or standard chemistry.
c
c  Revision 1.6  1998/11/09  17:22:31  ara
c  Included file errlog.cmd to introduce ierrlg variable
c  Directed error message to diag file and to log file (ierrlg)
c
c  Revision 1.5  1998/11/04  01:38:05  tcm
c  Enabled calculation of sumnox.
c
c  Revision 1.4  1996/06/14  22:57:18  tcm
c  diagnostics added by gem
c
c  Revision 1.3  1994/05/16  15:14:20  tcm
c  Additional diagnostics added.
c
c  Revision 1.2  94/02/10  20:54:51  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c  
c
c
c     driver for PIG chemistry routines
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'ctlflg.cmd'
      include 'comchem.cmd'
      include 'comd3.cmd'
      include 'pigpen.cmd'
      include 'rpm.cmd'
      include 'ppmcon.cmd'
      include 'pigdiag.cmd'
      include 'errlog.cmd' 
      common /idcell/icell,jcell,kcell,tmchm
      logical lpreac,llpt
c
c
      logical lerdch
c
      dimension contem(mxspec)
      dimension rcon(mxrads)
c
      llpt=.false.
c
c     return if no chemistry
c
      if (linert) return
      iskip = 0
c
c
c     check for species concentrations below lower bounds
c
      do 6 m=1,nospec
        do 5 nc=1,ncells
          if (pconc(nc,m+1).lt.bdmol(m)) pconc(nc,m+1)=bdmol(m)
    5   continue
    6 continue
c
c   calculate temperature dependent rates.
c
      call tdrate(nreact,tcell,pcell)
c
c   calculate photol factors
c
      call phot(time,nyr,nmth,nday,ig,jg,kg,zavght,factor,ldark)
c
c
c     calculate chemistry in plume cell by cell
c
      do 120 j=1,ncells
c
c     load contem witH all species beyond trac (species no.1)
c     (convert to ppm.)
c
        aam2c = cpfac*tcell/(273.*pcell)
        aac2m = pcell*273./(tcell*cpfac)
        kcell = j
        do 85 k=1,nospec
          contem(k)=pconc(j,k+1)*aam2c
   85   continue
c
c   calculate time step in hours for chemistry routines
c
c       dthr = dt/60.
c
        oh = 1.e-20
        ho2 = 1.e-20
        aoh = 1.e-20
        aho2t = 1.e-20
        cwc = 0.
c
c  set  radical concentrations (dfg, 05/17/99)
c
            do ll=1,mxrads
              rcon(ll)=1.0e-09
            end do
c
c   call steady state chemistry to get new concentrations
c
c       write(*,*) 'nospec, dt, contem, cwc, tcell, pcell:',
c    &         nospec, dt, contem, cwc, tcell, pcell
c
        if (.not.lpreac) then
          if(ldark) then
            call ssdark(contem,tcell,pcell,dt)
          else 
            call ssday(contem,tcell,pcell,dt)
          endif
        else
c
c    ...or call step4 to do full chem
c
          sumnox = contem(kno) + contem(kno2)
          if (sumnox.gt.10) then
            nskip = nskip + 1
c           iskip = iskip + 1
c           go to 90
          endif
          nchem = nchem + 1
c
c   we don't do irr on pig puffs yet, but modify call and pass
c   dummy parameters for consistency with the rest of the step4
c   calls in grid code.
c
          iflg = -9
          kdum = 1
c         llpt=.true.
          if (lchfast) then
            call step4f(nospec,nospec,dt,contem,lerror,tcell,rcon,llpt,
     &               0,iflg,kdum,aac2m)
          else
            call step4(nospec,nospec,dt,contem,lerror,tcell,rcon,llpt,
     &               0,iflg,kdum,aac2m)
          endif
          if (lerror) write(*,88) nsrc,tcell,pcell
   88     format(' In DOCHEM, just after call to STEP4.  nsrc = ',i10,
     &             /,'     tcell, pcell ',2e10.2)

c         write(*,89) nsrc
   89     format(' Just out of STEP4, nsrc = ',i10)
   90     continue
          lerdch = lerror
        endif
c
c     restore new concentrations to pconc array
c     -note-  load back into pconc making allowance for
c     TRAC (species no.1)
c     (convert back to umoles/m**3)
c
         do 110 k=1,nospec
           pconc(j,k+1)=contem(k)*aac2m
  110    continue
c
c     completion of plume cell loop
c
  120 continue
c      write(*,*)'SKIPPED ',iskip,' CALLS TO STEP4 BECAUSE NOX > 100 PPM'
c
c
c     obtain new plume averages
c
      call devcon(nspec,1,ncells)
c
  130 if(.not.lerdch) return
c
c  error occurred in dochem - dump puff arrays
c  (note that jcell was set to puff # in rpm)
c
      write(*,140) lerdch
      write(8,140) lerdch
      write(ierrlg,141) 1057,lerdch
  140 format('0PROGRAM ERROR IN PLUME SUBROUTINE DOCHEM'
     &,'LERDCH=',l5)
  141 format(i4,1x,'0PROGRAM ERROR IN PLUME SUBROUTINE DOCHEM',
     & 'LERDCH=',l5)
      write(*,'(a)') 'nplume, nsrcmbn, isrcmbn:'
      write(*,*) nplume(jcell), nsrcmbn(jcell), 
     &           (isrcmbn(jcell,mmm),mmm=1,nsrcmbn(jcell))
      write(*,'(a)') 'rfacb,rfacc,wfac:'
      write(*,*) (rfacb(mmm+1),rfacc(mmm),wfac(mmm),mmm=1,mxskn)
      write(*,'(a)') 'aml:'
      write(*,*) (aml(mmm,jcell),mmm=1,mxpspc)
      write(*,'(a)') 'osgy,osgz,oaml:'
      write(*,*) osgy(jcell),osgz(jcell),(oaml(mmm,jcell),mmm=1,mxpspc)
      write(*,'(a)') 'sigmy,osigmy,sigmz,osigmz:'
      write(*,*) sigmy,osigmy,sigmz,osigmz
      write(*,'(a)') 'nskins, emrate, wnd, tmprt:'
      write(*,*) nskins(jcell), (emrate(mmm,jcell),mmm=1,mxpspc),
     &                wnd(jcell), tmprt(jcell)
      write(*,'(a)') 'concs(mxpspc,mxskn,jcell):'
      write(*,*) ((concs(mmm,mms,jcell),mmm=1,mxpspc),mms=1,mxskn)
      write(*,'(a)') 'connul(mxpspc,mxskn,jcell):'
      write(*,*) ((connul(mmm,mms,jcell),mmm=1,mxpspc),mms=1,mxskn)
      write(*,'(a)') 'pconc(mxskn,mxpspc),yav(mxpspc),nspec:'
      write(*,*) ((pconc(mms,mmm),mms=1,mxskn),mmm=1,mxpspc),
     &           (yav(mmm),mmm=1,mxpspc),nspec
      write(*,'(a)') 'yaml(mxpspc):'
      write(*,*) (yaml(mxpspc),mmm=1,mxpspc)
      write(*,'(a)') 'sigy,synew,sigz,sznew:'
      write(*,*) sigy,synew,sigz,sznew
      return
      end
