      subroutine expand(mmx,mmy,mmz,nzg,ix,jy,kz,ug,vg,tg,tgs,
     &                  pg,hg,rkv,height,sigma,sigmav,rfacb,xlnpuf,
     &                  dstot,wdump,delx,dely,idump,iflg,idiag,stkdiam,
     &                  stkht,heff,n9)
 
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: expand.f,v $
c  Revision 1.6  1999/12/21  21:08:59  tcm
c  Cleaned up some format statements.
c
c  Revision 1.5  1996/06/14  22:58:16  tcm
c  see comments below
c
c  Revision 1.4  1994/07/13  13:25:47  tcm
c  Impose minimum value on wshr2 to avoid floating pt exception
c  CAE added a check for zero plume rise to avoid assigning an initial
c  plume depth of zero.
c
c  Revision 1.3  94/05/16  15:12:00  tcm
c  Plume expansion revised by CAE.
c  Initial plume expansion calculated and F stability eliminated.
c  Additional optional diagnostics added.
c  
c  Revision 1.2  94/02/10  20:55:19  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c  
c  Modified checks on tgrad small for calculation of Ri
c  check now looks at absolute value of tgrad --- 96/03/27  gem
c
c
c-----performs the following:
c     (1) calculates stability class based on bulk richardson number 
c         across the plume depth
c     (2) determines sigma(y) and sigma(z)
c     (3) limits plume growth to pbl depth or domain depth, whichever 
c         is lower
c     (4) limits plume growth to volume of current gridcell (and set 
c         for dumping if it exceeds volume)
c     (5) move puff centerline when vertical puff expansion is not 
c         allowed equally in both positive and negative directions
c
c     c emery    11/15/91
c
c     fixed value of minbot for case rkv lt 1e+4 (cae, 3/26/93)
c
 
      include 'parm.cmd'
      include 'advec.cmd'
      dimension ug(mmx,mmy,mmz), vg(mmx,mmy,mmz), hg(mmx,mmy,mmz), 
     &          rkv(mmx,mmy,mmz), pg(mmx,mmy,mmz), tg(mmx,mmy,mmz)
      dimension u(mxz), v(mxz), t(mxz), zmid(mxz)
      data g /9.8/, gamma /0.286/, dpdz /-0.1/, pi /3.1415927/

c-----convert units to mks, load wind speed & potential temperature
c     arrays, and find pbl depth

      plmht = 1000.*height 
      do 10 k = 1,nzg
        if (lstagr) then
          u(k) = 16.67*(ug(ix,jy,k) + ug(ix-1,jy,k))/2.
          v(k) = 16.67*(vg(ix,jy,k) + vg(ix,jy-1,k))/2.
        else
          u(k) = 16.67*ug(ix,jy,k)
          v(k) = 16.67*vg(ix,jy,k)
        endif
        t(k) = tg(ix,jy,k)*(1000./(1013.*pg(ix,jy,k)))**gamma
   10 continue
      
c-----find layer midpoint heights, find surface temp & press

      zmid(1) = 1000.*hg(ix,jy,1)/2.
      do 20 k = 2,nzg
        zmid(k) = 1000.*(hg(ix,jy,k)+hg(ix,jy,k-1))/2.
   20 continue
      ps = 1013.*pg(ix,jy,1) - dpdz*zmid(1)
      ts = tgs*(1000./ps)**gamma

c-----load old values of sigma(y) and sigma(z)
c     For new puffs (IFLG=1), set both initial sigmas to sigmay, calculated
c     using distance between stack top and final plume rise, and mean Ri
c     over that distance 

      if (iflg.eq.0) then
        oldsy = sigma
        oldsz = sigmav
      else
        oldsy = stkdiam
        oldsz = stkdiam
        if (stkht.eq.heff) goto 25
        top = heff
        if (top.gt.1000.*hg(ix,jy,nzg)) top = 1000.*hg(ix,jy,nzg) 
        do 301 k=1,nzg
          if (top .lt. zmid(k)) then
            ktop = k
            go to 401
          endif
  301   continue
        ktop = nzg
  401   continue
        bttm = stkht
        if (bttm.lt.0.) bttm = 0.
        if (bttm .gt. zmid(nzg)) kbot = nzg-1
        do 501 k=nzg-1,1,-1
          if (bttm .gt. zmid(k)) then
            kbot = k
            go to 601
          endif
  501   continue
        kbot = 0
  601   continue
        depth = top - bttm
c       if (idiag.ne.0.and.n9.eq.1) then
c         write(9,*)'top,bottom: ',top,ktop,bttm,kbot
c         write(9,*)'depth: ',depth
c       endif

c-----calculate shear & potential temperature gradient across 
c     plume rise depth
 
        if (ktop-1.eq.0) then
          uup = u(ktop)*top/zmid(ktop)
          vup = v(ktop)*top/zmid(ktop)
          tup = (t(ktop)*top + ts*(zmid(ktop)-top)) / zmid(ktop)
        else
          uup = (u(ktop)*(top-zmid(ktop-1)) +
     &           u(ktop-1)*(zmid(ktop)-top)) /
     &           (zmid(ktop)-zmid(ktop-1))
          vup = (v(ktop)*(top-zmid(ktop-1)) +
     &          v(ktop-1)*(zmid(ktop)-top)) /
     &           (zmid(ktop)-zmid(ktop-1))
          tup = (t(ktop)*(top-zmid(ktop-1)) +
     &         t(ktop-1)*(zmid(ktop)-top)) /
     &         (zmid(ktop)-zmid(ktop-1))
        endif
        if (kbot.eq.0) then
          udn = u(kbot+1)*bttm/zmid(kbot+1)
          vdn = v(kbot+1)*bttm/zmid(kbot+1)
          tdn = (t(kbot+1)*bttm + ts*(zmid(kbot+1)-bttm)) / zmid(kbot+1)
        else
          udn = (u(kbot+1)*(bttm-zmid(kbot)) +
     &           u(kbot)*(zmid(kbot+1)-bttm)) /
     &           (zmid(kbot+1)-zmid(kbot))
          vdn = (v(kbot+1)*(bttm-zmid(kbot)) +
     &         v(kbot)*(zmid(kbot+1)-bttm)) /
     &         (zmid(kbot+1)-zmid(kbot))
          tdn = (t(kbot+1)*(bttm-zmid(kbot)) +
     &           t(kbot)*(zmid(kbot+1)-bttm)) /
     &           (zmid(kbot+1)-zmid(kbot))
        endif

        ushr = (uup - udn)/depth
        vshr = (vup - vdn)/depth
C
C *** TCM MOD (06/07/94) - To correct problem (floating point exception)
C ***                      that occurs when wshr2=0 (this occurs when two
C ***                      levels have the same u and v wind components).
C ***
C *** wshr2 = ushr**2 + vshr**2
c
c  set minimum value on wshr2 (tcm, 6/7/94)
        wshr2 = amax1(ushr**2 + vshr**2,1.E-20)
        tgrad = (tup - tdn)/depth
        thbar = (tup + tdn)/2. 
c       if (idiag.ne.0.and.n9.eq.1) then
c          write(9,*)'Top wind: ',uup,vup
c          write(9,*)'Bot wind: ',udn,vdn
c          write(9,*)'shear**2: ',wshr2
c          write(9,*)'Top temp: ',tup
c          write(9,*)'Bot temp: ',tdn
c          write(9,*)'DT, Tbar: ',tgrad,thbar
c       endif

c-----calculate critical and bulk richardson number
  
        ric = 0.115*(depth*100.)**0.175
c       if (tgrad.gt.0..and.tgrad.lt.0.001) tgrad = 0.
        if (abs(tgrad) .lt.0.0001) tgrad = 0.
        ri = g*tgrad/(thbar*wshr2)
c       if (idiag.ne.0.and.n9.eq.1) write(9,*)'ric, ri:',ric,ri

c-----determine stability class based on richardson number
       
        if (ri.lt.-1.) then
          kstab = 2
        elseif (ri.lt.0.) then
          kstab = 3
        elseif (ri.lt.ric) then
          kstab = 4
        else
          kstab = 5
c       elseif (ri.lt.10.) then
c         kstab = 5
c       else
c         kstab = 6
        endif
        oldsy = sigmay(depth,oldsy,kstab,1)
        oldsz = oldsy
c       if (idiag.ne.0.and.n9.eq.1)
c    &     write(9,*)'EXPAND: initial sigma from plume rise: ',n9,oldsy
      endif
c     if (idiag.ne.0.and.n9.eq.1) 
c    &   write(9,*) ' in expand, oldsy = ',n9,oldsy

c-----find plume depth, top & bottom heights

   25 top = plmht + rfacb*oldsz
      if (top.gt.1000.*hg(ix,jy,nzg)) top = 1000.*hg(ix,jy,nzg) 
      do 30 k=1,nzg
        if (top .lt. zmid(k)) then
          ktop = k
          go to 40
        endif
   30 continue
      ktop = nzg
   40 continue
      bttm = plmht - rfacb*oldsz
      if (bttm.lt.0.) bttm = 0.
      if (bttm .gt. zmid(nzg)) kbot = nzg-1
      do 50 k=nzg-1,1,-1
        if (bttm .gt. zmid(k)) then
          kbot = k
          go to 60
        endif
   50 continue
      kbot = 0
   60 continue
      depth = top - bttm
c     if (idiag.ne.0.and.n9.eq.1) then
c        write(9,*)'top,bottom: ',top,ktop,bttm,kbot
c        write(9,*)'depth: ',depth
c     endif

c-----move centerline height if top and/or bttm were limited by top of 
c     domain and/or ground, respectively

      if (iflg.eq.0) then
        oldht = height
        height = (top + bttm)/2000.
        if (abs(height-oldht).gt.0.001) then
          do 70 k = 1,nzg
            if (height.lt.hg(ix,jy,k)) then
              kz = k
              goto 71
            endif
   70     continue
   71     continue
c         if (idiag.ne.0.and.n9.eq.1) 
c    &     write(9,*)'Had to adjust centerline from',oldht,' to',height
        endif 
      endif

c-----calculate shear & potential temperature gradient across 
c     plume depth
 
      if (ktop-1.eq.0) then
        uup = u(ktop)*top/zmid(ktop)
        vup = v(ktop)*top/zmid(ktop)
        tup = (t(ktop)*top + ts*(zmid(ktop)-top)) / zmid(ktop)
      else
        uup = (u(ktop)*(top-zmid(ktop-1)) +
     &         u(ktop-1)*(zmid(ktop)-top)) /
     &         (zmid(ktop)-zmid(ktop-1))
        vup = (v(ktop)*(top-zmid(ktop-1)) +
     &        v(ktop-1)*(zmid(ktop)-top)) /
     &         (zmid(ktop)-zmid(ktop-1))
        tup = (t(ktop)*(top-zmid(ktop-1)) +
     &         t(ktop-1)*(zmid(ktop)-top)) /
     &         (zmid(ktop)-zmid(ktop-1))
      endif
      if (kbot.eq.0) then
        udn = u(kbot+1)*bttm/zmid(kbot+1)
        vdn = v(kbot+1)*bttm/zmid(kbot+1)
        tdn = (t(kbot+1)*bttm + ts*(zmid(kbot+1)-bttm)) / zmid(kbot+1)
      else
        udn = (u(kbot+1)*(bttm-zmid(kbot)) +
     &         u(kbot)*(zmid(kbot+1)-bttm)) /
     &         (zmid(kbot+1)-zmid(kbot))
        vdn = (v(kbot+1)*(bttm-zmid(kbot)) +
     &         v(kbot)*(zmid(kbot+1)-bttm)) /
     &         (zmid(kbot+1)-zmid(kbot))
        tdn = (t(kbot+1)*(bttm-zmid(kbot)) +
     &         t(kbot)*(zmid(kbot+1)-bttm)) /
     &         (zmid(kbot+1)-zmid(kbot))
      endif

      ushr = (uup - udn)/depth
      vshr = (vup - vdn)/depth
      wshr2 = amax1(ushr**2 + vshr**2,1.E-20)
      tgrad = (tup - tdn)/depth
      thbar = (tup + tdn)/2. 
c     if (idiag.ne.0.and.n9.eq.1) then
c        write(9,*)'Top wind: ',uup,vup
c        write(9,*)'Bot wind: ',udn,vdn
c        write(9,*)'shear**2: ',wshr2
c        write(9,*)'Top temp: ',tup
c        write(9,*)'Bot temp: ',tdn
c        write(9,*)'DT, Tbar: ',tgrad,thbar
c     endif

c-----calculate critical and bulk richardson number

      ric = 0.115*(depth*100.)**0.175
c     if (tgrad.gt.0..and.tgrad.lt.0.001) tgrad = 0.
      if (abs(tgrad).lt.0.0001) tgrad = 0.
      ri = g*tgrad/(thbar*wshr2)
c     if (idiag.ne.0.and.n9.eq.1) write(9,*)'ric, ri:',ric,ri

c-----determine stability class based on richardson number
       
      if (ri.lt.-1.) then
        kstab = 2
      elseif (ri.lt.0.) then
        kstab = 3
      elseif (ri.lt.ric) then
        kstab = 4
      else
        kstab = 5
c     elseif (ri.lt.10.) then
c       kstab = 5
c     else
c       kstab = 6
      endif
c     if (idiag.ne.0.and.n9.eq.1) write(9,*)'kstab:',kstab

c
c  find new sigma and sigmav 
c
      dist = 1000.*dstot
      sigma = sigmay(dist,oldsy,kstab,1)
      sigmav = sigmaz(dist,oldsz,kstab,1)
c     if (idiag.ne.0.and.n9.eq.1) write(9,*)'sigma,sigmav:',sigma,sigmav
c
c  limit puff's vertical dimension to local mixed depth (whether surface-
c  based or aloft) -- maximum depth is domain depth
c
      maxtop = nzg
      minbot = 0
      if (ri.gt.0.) goto 111
c
c  how far up does mixed layer go above puff centerline?
c
      if (kz.ne.nzg) then
        if (rkv(ix,jy,kz).gt.1.e4) then
          do 100 k = kz+1,nzg 
            if (rkv(ix,jy,k).lt.1.e4) then
              maxtop = k
              goto 101
            endif
  100     continue
        else
          maxtop = kz
        endif
      endif 
 101  continue
c
c  how far down does mixed layer go below puff centerline?
c
      if (kz.ne.1) then
        if (rkv(ix,jy,kz-1).gt.1.e4) then
          if (kz.eq.2) goto 111
          do 110 k = kz-2,1,-1
            if (rkv(ix,jy,k).lt.1.e4) then
              minbot = k-2
              goto 111
            endif
  110     continue
        else
          minbot = kz-1
        endif
      endif 
  111 continue
c
c  adjust height of centerline and sigmav if growth of puff depth is to be 
c  limited to a maximum value. note: do not allow sigmav < oldsz !
c
      if (minbot .eq. 0) then
        bttmin = 0.
      else
        bttmin = 1000.*hg(ix,jy,minbot)
      endif
      topmax = 1000.*hg(ix,jy,maxtop) 
      depthmax = topmax - bttmin
      if (depthmax.lt.2.*rfacb*sigmav .and. 
     &    depthmax.gt.2.*rfacb*oldsz) then
        sigmav = depthmax/(2.*rfacb)
        height = (topmax + bttmin)/2000.
        do 120 k = 1,nzg
          if (height.lt.hg(ix,jy,k)) then
            kz = k
            goto 121
          endif
  120   continue
  121   continue
      endif 
c
c  check to make sure puff volume is no larger than a factor of wdump/2
c  bigger than current gridcell volume.  if it is, set volume to that fraction,
c  and flag for dumping. note: ratio of sigma/sigmav is kept constant
c
c     if (idiag.ne.0.and.n9.eq.1) 
c    &   write(9,*) ' in expand, sigma and sigmav: ',sigma,sigmav
      a = rfacb*sigma/1000.
      b = rfacb*sigmav/1000.
      abrat = a/b
      puffvol = xlnpuf*pi*a*b
c     if (idiag.ne.0.and.n9.eq.1) 
c    &   write(9,*) ' ix,jy,kz,mmz ',ix,jy,kz,mmz
      delz = hg(ix,jy,kz) 
      if (kz.gt.1) delz = hg(ix,jy,kz) - hg(ix,jy,kz-1)
c     if (idiag.ne.0.and.n9.eq.1) 
c    &   write(9,*) ' wdump, delx, dely, delz ',wdump,delx,dely,delz
      gridvol = (wdump/2.)*delx*dely*delz
      if (puffvol.gt.gridvol) then
        b = sqrt( gridvol/(abrat*xlnpuf*pi) )
        a = abrat*b
        idump = 4
        sigma  = a*1000./rfacb
        sigmav = b*1000./rfacb
c       if (idiag.ne.0.and.n9.eq.1) 
c    &      write(9,*) 'in expand, a, b, rfacb, sigma: ',
c    &                                a,b,rfacb,sigma
c       if (idiag.ne.0.and.n9.eq.1) 
c    &      write(9,*) ' gridvol, abrat, xlnpuf ',
c    &                                gridvol,abrat,xlnpuf
      endif
      if (sigma.lt.oldsy .or. sigmav.lt.oldsz) then
        sigma = oldsy
        sigmav = oldsz
        if (idiag.ne.0) write(*,200) puffvol, gridvol
        if (idiag.ne.0) write(8,200) puffvol, gridvol
c       if (idiag.ne.0.and.n9.eq.1) write(9,200) puffvol, gridvol
      endif

      return

 200  format('             ********** WARNING! *************',/,
     &       'Puff volume is:',e10.3,'; larger than grid volume',e10.3,
     &       /,'         Adjusted sigmay and/or sigmaz is less than',/,
     &       '         value during previous time step: resetting',/,
     &       '         to the old values')

      end
