      subroutine fgrtrp(mxc,myc,nx,ny,nz,acrs,
     &                  mxf,myf,nzfin,afine,htf,ifine,itype,idebug)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: fgrtrp.f,v $
c  Revision 1.4  1996/06/14  22:59:03  tcm
c  corrected height calculation for vertical nesting
c
c  Revision 1.3  1995/04/26  00:39:30  tcm
c  	rewritten to allow more complex vertical nesting.
c
c  Revision 1.2  94/02/10  20:55:48  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c  
c
c
c
c  interpolates coarse grid data onto fine grid for one of
c  the fine grids selected by ifine.
c  (tcm, 9/1990) 
c
c  ones digit of itype selects type of interpolation for horizontal:
c
c        = 0 will result in a bilinear interpolation
c
c        = 1 will do a bilinear interpolation and then adjust values
c  such that the average of all fine grid values in a coarse grid cell
c  is equal to the value in the coarse grid cell
c
c  tens digit of itype selects type of interpolation for vertical:
c
c        = 0 assumes values at layer interfaces, linear interpolation
c            to additional layer interfaces, assume 0. at surface.
c
c        = 1 assumes values at layer middle, linear interpolation to
c            get values at new layer middles, assume same value from
c            middle of coarse grid layer one down to surface.
c
c        = 2 means that the value of nzfin is not actually vertical
c            levels and vertical interpolation should therefore be
c            skipped
c
      include 'parfin.cmd'
      include 'parm.cmd'
      include 'deffin.cmd'
      dimension acrs(mxc,myc,nz), afine(mxf,myf,nzfin)
      dimension htf(mxf,myf,nzfin)
c
c  local arrays to ease the pain of the vertical interpolation
c
      dimension vcrs(mxz), vfine(mxfz), hcv(mxz)
c
c  fine grids include an extra row/column on top,left, 
c  bottom, and right of fine cells to allow for boundary
c  concentrations.  Simulated area is therefore exactly an
c  integral number of coarse grid cells.  niggle is a value
c  that tells the number of fine cells that we are short of
c  being at the mid point of a coarse cell at the start of the
c  overall fine grid.  It is used to help pick the correct cell
c  cells to use in interpolation.
c
c  Bit two of idebug on (i.e., the twos location) will turn
c  on debug writes in fgrtrp.
c
      idiag = and(idebug,2)
c
      niggle = 1+(nhf(ifine)+1)/2
c
      if (idiag .ne. 0) then
        write(*,'(a,i4)') ' niggle: ',niggle
      endif
c
c  split up itype into vertical and horizontal values
c
      ihtype = mod(itype,10)
      ivtype = itype/10
c
c  calculate interpolation factors (nhf of them)
c
      if (mod(nhf(ifine),2) .eq. 0) then
        ff1(1) = 0.5/float(nhf(ifine))
      else
        ff1(1) = 1./float(nhf(ifine))
      endif
      ff0(1) = 1. - ff1(1)
c
      if (idiag .ne. 0) then
        write(*,'(a,2i5)') 'ihtype, ivtype: ',ihtype,ivtype
        write(*,'(a,2e11.4)') ' ff0   ff1: ',ff0(1),ff1(1)
      endif
c
      do 20 n=2,nhf(ifine)
        ff1(n) = ff1(n-1) + 1./float(nhf(ifine))
        ff0(n) = 1. - ff1(n)
c
        if (idiag .ne. 0) then
          write(*,'(a,2e11.4)') ' ff0   ff1: ',ff0(n),ff1(n)
        endif
c
   20 continue
c
c  first do horizontal interpolation at each level
c  (note that we leave gaps between levels on the fine
c  grid to interpolate vertically later)
c 
      do 100 kc=1,nz
        if (ivtype .eq. 2) then
          kf = kc
        else
          kf = kvmap(kc,ifine)
        endif
        do 60 j=jyfb(ifine),jyfe(ifine)+1
          jef = (j - jyfb(ifine))*nhf(ifine) + niggle
          jbf = max(1,jef-nhf(ifine)+1)
          do 50 i=ixfb(ifine),ixfe(ifine)+1
            ief = (i - ixfb(ifine))*nhf(ifine) + niggle
            ibf = max(1,ief-nhf(ifine)+1)
            do 40 jf=jbf,jef
              if (jf .gt. nfy(ifine)) go to 40
              nn = nhf(ifine) - jef + jf
              x0 = acrs(i-1,j-1,kc)*ff0(nn) + acrs(i-1,j,kc)*ff1(nn)
              x1 = acrs(i,j-1,kc)*ff0(nn) + acrs(i,j,kc)*ff1(nn)
              do 30 if=ibf,ief
                if (if .gt. nfx(ifine)) go to 30
                nn = nhf(ifine) - ief + if
                afine(if,jf,kf) = x0*ff0(nn) + x1*ff1(nn)
c
                if (idiag .ne. 0) then
                  if ( (i .eq. ixfb(ifine) .or. i .eq. ixfb(ifine)+1)
     &                .and.
     &                (j .eq. jyfb(ifine) .or. j .eq. jyfb(ifine)+1) )
     &                then
                    write(*,'(a,/,1x,i4,5e11.4,/,5x,a,/,5x,11i4)') 
     &               ' nn, ff0, ff1, x0, x1, afine',
     &               nn, ff0(nn), ff1(nn), x0, x1, afine(if,jf,kf),
     &               'ibf, ief, jbf, jef, if, jf, kf, i, j, kc, ifine',
     &               ibf, ief, jbf, jef, if, jf, kf, i, j, kc, ifine
                  endif
                endif
c
   30         continue
   40       continue
   50     continue
   60   continue
  100 continue
c
c  adjust interpolated data to average to coarse cell value
c  (if ihtype = 1)
c 
      if (ihtype .eq. 1) then
c
        fn2i = 1./float(nhf(ifine)*nhf(ifine))
        do 290 kc=1,nz
          if (ivtype .eq. 2) then
            kf = kc
          else
            kf = kvmap(kc,ifine)
          endif
          if (idiag .ne. 0) then
            write(*,'(a,4i5)') 'kc, kf, nadd, nz: ',kc,kf,nadd,nz
          endif
          do 260 j=jyfb(ifine),jyfe(ifine)
            jef = (j - jyfb(ifine) + 1)*nhf(ifine) + 1
            jbf = jef-nhf(ifine)+1
            do 250 i=ixfb(ifine),ixfe(ifine)
              ief = (i - ixfb(ifine) + 1)*nhf(ifine) + 1
              ibf = ief-nhf(ifine)+1
              sum = 0.
              do 220 jf=jbf,jef
                do 210 if=ibf,ief
                  sum = sum + afine(if,jf,kf)
  210           continue
  220         continue
              avg = sum*fn2i
              if (avg .ne. 0.) then
                fcor = acrs(i,j,kc)/avg
              else
                fcor = 1.
              endif
              if (idiag .ne. 0) then
                write(*,'(a,e12.5,2x,e12.5,2x,e12.5,2x,e12.5)')
     &              'sum, fn2i, avg, fcor: ', sum, fn2i, avg, fcor
                write(*,'(a,3i5,e12.5)') 'i, j, kc, acrs: ',
     &              i, j, kc, acrs(i,j,kc)
              endif
              do 240 jf=jbf,jef
                do 230 if=ibf,ief
                  afine(if,jf,kf) = fcor*afine(if,jf,kf)
                  if (idiag .ne. 0) then
                    if (if .gt. 2 .and. if .lt. 5 .and.
     &                  jf .gt. 2 .and. jf .lt. 5) then
                      write(*,'(a,3i5,e12.5)') 'if,jf,kf,afine: ',
     &                      if, jf, kf, afine(if,jf,kf)
                    endif
                  endif
  230           continue
  240         continue
  250       continue
  260     continue
  290   continue
c
      endif
c
c   write out first few values of both coarse grid and fine
c   grid for diagnostics
c
      if (idiag .ne. 0) then
        write(*,'(a,5e11.4,/,a,5e11.4,/,a,10e11.4,/,a,10e11.4)')
     &      ' Coarse:  ',
     &      (acrs(i,jyfb(ifine),1),i=ixfb(ifine),ixfb(ifine)+4),
     &      '          ',
     &      (acrs(i,jyfb(ifine)+1,1),i=ixfb(ifine),ixfb(ifine)+4),
     &      ' Fine:  ',
     &      (afine(i,2,1),i=2,11),
     &      '        ',
     &      (afine(i,3,1),i=2,11)
      endif
c
      if (nzfin .eq. nz) return
c
c  interpolate to tops of intervening layers
c
      if (ivtype .eq. 0) then
        kf = 0
        do 350 kc=1,nz
          f1 = 0.
          kflow = kf
          kf = kvmap(kc,ifine)
          nl = kf - kflow - 1
          do 330 nn=1,nl
            kkf = kflow + nn
            do 320 jf=1,nfy(ifine)
              do 310 if=1,nfx(ifine)
                if (kc .gt. 1) then
                  a1 = afine(if,jf,kflow)
                  htm1 = htf(if,jf,kflow)
                else
                  a1 = afine(if,jf,kf)
                  htm1 = 0.
                endif
                f1 = (htf(if,jf,kkf)-htm1)/(htf(if,jf,kf)-htm1)
                f0 = 1. - f1
                afine(if,jf,kkf) = a1*f0 + afine(if,jf,kf)*f1
  310         continue
  320       continue
  330     continue
  350   continue
c
      else if (ivtype .eq. 1) then
c
c   interpolate to middles of layers
c
        do 470 j=1,nfy(ifine)
          do 460 i=1,nfx(ifine)
            do 410 k=1,nz
              kf = kvmap(k,ifine)
              vcrs(k) = afine(i,j,kf)
              if (k .gt. 1) then
                kfm1 = kvmap(k-1,ifine)
                hcv(k) = 0.5*(htf(i,j,kf)+htf(i,j,kfm1))
              else
                hcv(k) = 0.5*htf(i,j,kf)
              endif
  410       continue
            do 415 kf = 1,kvmap(1,ifine)
              vfine(kf) = vcrs(1)
  415       continue
            if (nz .le. 1) go to 460
            do 420 kf=kvmap(nz-1,ifine)+1,nzfin
              vfine(kf) = vcrs(nz)
  420       continue
            do 440 k=2,nz
              if (k .gt. 2) then
                kfbeg = kvmap(k-2,ifine) + 1
              else
                kfbeg = 1
              endif
              do 430 kf=kfbeg,kvmap(k,ifine)
		if ( kf .gt. 1 ) then
		   hvm = 0.5 * (htf(i,j,kf) + htf(i,j,kf-1))
		else
		   hvm = 0.5 * htf(i,j,kf)
		endif
                if (hvm .gt. hcv(k)) go to 430
                if (hvm .le. hcv(k-1)) go to 430
                f1 = (hvm-hcv(k-1))/(hcv(k)-hcv(k-1))
                f0 = 1. - f1
                vfine(kf) = f0*vcrs(k-1) + f1*vcrs(k)
  430         continue
  440       continue
            do 450 kf=1,nzfin
              afine(i,j,kf) = vfine(kf)
  450       continue
  460     continue
  470   continue
c
      else
c
        return
c
      endif
      if (idiag .ne. 0) then
        write(*,'(a,5e11.4,/,a,5e11.4,/,a,10e11.4,/,a,10e11.4)')
     &      ' After vert, Coarse:  ',
     &      (acrs(i,jyfb(ifine),1),i=ixfb(ifine),ixfb(ifine)+4),
     &      '                      ',
     &      (acrs(i,jyfb(ifine)+1,1),i=ixfb(ifine),ixfb(ifine)+4),
     &      ' After vert, Fine:  ',
     &      (afine(i,2,1),i=2,11),
     &      '                    ',
     &      (afine(i,3,1),i=2,11)
      endif
c
      return
      end
