      subroutine fstep(deltat,idebug)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: fstep.f,v $
c  Revision 1.3  1995/04/26  00:40:11  tcm
c  	due to more complex horizontal nesting, check all adjacent grids to ensure
c  	proper synchronization
c
c  Revision 1.2  94/02/10  20:56:20  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c  
c
c
c
c  calculates maximum time step and number of time steps
c  to be used on fine grid in completing one of the
c  coarse grid size steps.  (tcm, 2/12/91)
c  
c  Vars calculated are:
c
c          ndftot - total number of time steps on fine grid
c                   to complete one coarse grid step
c          ndfmax - maximum number of steps to take in a
c                   pass (this will be updated in aaamain)
c          ndfcom - number of steps completed (initialized to
c                   zero here)
c          deltf  - time step size on fine grid
c
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'com1f.cmd'
      include 'comuam.cmd'
      include 'ngmap.cmd'
c
c  Third bit of idebug (i.e., 4's place) will turn on diagnostic
c  writes in fstep
c
      idiag = and(idebug,4)
c
c   loop over all fine grids
c
      do 300 ifine=1,numfin
c
c  calculate maximum absolute x and y component of wind speed within
c  this fine grid, from coarse grid values if fine grid winds will
c  be interpolated or from fine grid if fine grid winds have already
c  been read in.
c
        if (ifmdf(ifine) .gt. 0) then
          call fgmax(nfx(ifine),nfy(ifine),nfz(ifine),
     &               uf(1,1,ifine),vf(1,1,ifine),umax,vmax)
        else
          umax = -999.
          vmax = -999.
          do 100 k=1,nlayer
            do 90 j=jyfb(ifine),jyfe(ifine)
              do 80 i=ixfb(ifine),ixfe(ifine)
                umax = amax1(abs(u(i,j,k)),umax)
                vmax = amax1(abs(v(i,j,k)),vmax)
   80         continue
   90       continue
  100     continue
        endif
c
        delxmn = 1.e20
        do 101 j=1,nfy(ifine)
          delxmn = amin1(delxmn,delfx(j,ifine))
  101   continue
c
c   calculate maximum time step based on wind speed and grid size
c
        dtmax = 0.5*amin1(delxmn/umax,delfy(ifine)/vmax)
c
c   calculate limitation due to horizontal diffusivity
c  
        diff=0.0
c
c...    Find the minimum horizontal grid resolution to calculate
c...    time step later
        if(delxmn .gt. delfy(ifine))then
          horzds=delfy(ifine)
        else
          horzds=delxmn
        endif
c
        horz2i = 1./horzds**2
        do 200 k=1,nlayer
          do 190 j=jyfb(ifine),jyfe(ifine)
            do 180 i=ixfb(ifine),ixfe(ifine)
              diff=AMAX1(diff,kh(I,J,K)*horz2i)
  180       continue
  190     continue
  200   continue
c
        if (diff .gt. 0.) then
          dt2=0.15/diff
        else
          dt2=1.E20
        endif
c
c     determine maximum deltat for this time step
c
        dtmax = amin1(dtmax,dt2)
c
c     make sure that deltf goes into next delta t evenly
c
        if (ifglvl(ifine) .eq. 1) then
          dtchk = deltat
        else
          dtchk = deltf(ifgptr(ifine))
        endif
        ndftot(ifine) = int(dtchk/dtmax) + 1
        ndfmax(ifine) = ndftot(ifine)
        ndfcom(ifine) = 0
        deltf(ifine) = dtchk/float(ndftot(ifine))
        if (ifglvl(ifine) .gt. 1) then
          ndftot(ifine) = ndftot(ifine)*ndftot(ifgptr(ifine))
          ndfmax(ifgptr(ifine)) = 1
        endif
c
c   check all edges of this grid.  if any edge touches another
c   fine grid edge, then we must limit to one step per pass
c
        do 240 iedg=1,2
          do 230 i=2,nfx(ifine)-1
            if (igloc(i,iedg,ifine) .ne. ifgptr(ifine)) then
              ndfmax(ifine) = 1
            endif
  230     continue
  240   continue
        do 260 iedg=3,4
          do 250 i=2,nfy(ifine)-1
            if (igloc(i,iedg,ifine) .ne. ifgptr(ifine)) then
              ndfmax(ifine) = 1
            endif
  250     continue
  260   continue
c
        if (idiag .ne. 0) then
          write(*,'(a,/,a,/,7e10.4,i4)')
     &      ' fstep calculates: umax, vmax, diff, dt2, dtmax, deltat,',
     &      ' deltf, ndtf', umax, vmax, diff, dt2, dtmax, deltat,
     &      deltf(ifine), ndftot(ifine)
        endif
c
  300 continue
c
      return
      end
