      subroutine prtdep(rsize,rmass,nbin,ustar,zo,rho,temp,
     &ra,vmean)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: partdep.f,v $
c  Revision 1.2  1994/02/10  20:58:31  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c
c
c
c
c**********************************************************************
c
c-----this program estimates a mass weighted mean deposition velocity
c-----for a defined particulate distribution.  the method that is used
c-----is similar to that used in adom.  the method assumes no
c-----resuspension from vegetation surfaces.
c
c-----the model inputs are the following:
c
c-----rsize = array of particle sizes (microns)
c-----rmass = array of mass fraction for each particle size
c-----nbin = number of particle size bins
c-----ustar = the true neutral friction velocity (used in laminar
c-----        resistance estimates) (m/s)
c-----zo = roughness length (m)
c-----rho = density of the particles (g/m3)
c-----temp = ambient temperature (deg k)
c-----ra = aerodynamic resistance  (s/m)
c
c-----the output of this routine is:
c-----vmean = the mass mean deposition velocity (m/s)
c
c-----this routine was written by gem 1990
c
c**********************************************************************
c
      data a1,a2,a3,xmean,visc/1.257,0.40,0.55,6.53e-08,1.5e-05/
      dimension rmass(nbin),rsize(nbin)
      dimension vd(20),rd(20),vg(20),eff(20)
c
c-----initialize arrays to zero
c
      xf = 1.0e-06 
      do 5 i = 1,20
        vg(i) = 0.0
        vd(i) = 0.0
        rd(i) = 0.0
    5 continue
c
c-----estimates the settling velocity
c
      do 10 i = 1,nbin
        d = rsize(i)*xf
        call sedrat(d,rho,vsed)
        vg(i) = vsed
   10 continue
c
c-----estimates the reynolds adjustment parameter
c
      re = ustar*zo/visc
c     b1 = re**0.25
      b1 = 1.0
c
c-----estimates the stokes law coeff for the critical radius dcrit
c
      dcrit = 3.0e-07
      call sedrat(dcrit,rho,vsed)
      call stoke(dcrit,vsed,ustar,st)
      call shmidt(dcrit,temp,sc)
      sc = 1.0/(b1*sc**0.666)
      b2 = sc/st
c
c-----estimates the schmidt and stokes numbers and uses them
c-----to estimate the depostion velocity
c
      do 20 i = 1,nbin
        d = rsize(i)*xf
        vsed = vg(i)
        call stoke(d,vsed,ustar,st)
        call shmidt(d,temp,sc)
        sc = 1.0/(b1*sc**0.666)
        st = b2*st
        eff(i) = 0.25*(sc + st)
        rd(i) = 1.0/(ustar*eff(i))
        vd(i) = vg(i) + 1.0/(ra + rd(i) + ra*rd(i)*vg(i))
   20 continue
c
c-----now estimates the mass weighted mean velocity
c
      sum = 0.0
      do 40 i = 1,nbin
        sum = sum + rmass(i)*vd(i)
   40 continue
      vmean = sum
      return
      end
