      subroutine puffdmp(mmx,mmy,mmz,npuff,ifgnow,ig,jg,kg,
     &                   ifn,jfn,kfn,hg,dy,deltat,idiag)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: puffdmp.f,v $
c  Revision 1.6  1999/12/21  19:28:21  tcm
c  Renamed com3 include file to comd3.
c
c  Revision 1.5  1998/11/11  17:46:41  ara
c  Included errlog.cmd file to introduce ierrlg variable
c  Directed error messages to diag file and to log file (ierrlg)
c
c  Revision 1.4  1996/06/14  23:02:07  tcm
c  see comments below
c
c  Revision 1.3  1995/04/26  00:43:02  tcm
c  	added code to correctly calculate plume location when in lat/lon coords
c
c  Revision 1.2  94/02/10  20:59:16  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c  
c
c

c-----this subroutine does the following:
c     (1) determines whether to dump onion skin(s), based on grid 
c         considerations
c     (2) calculates the mass to be dumped to uam-v grid (in terms of emission 
c         rate, mol/min)
c     (3) calculates onion-skin volume (km^3)
c     (4) loads total dump time for each grid (min) 
c     (4) determines cell/grid information for coarse and all fine grids to 
c         receive the dump.
c
c     c emery, 11/20/91
c
c---- modified to subtract null puff (ambient only) concentrations
c     from concs to get condmp (tcm, 1/22/92)
c
c---- fixed subscript for rfacb to be last skin instead of npuff
c     at around line 86 (tcm, 6/16/93)
c
c---- fixed calculation of kdmp for parent grids at around line 298
c     (gem, 12/26/95)

      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'comuam.cmd'
      include 'comd3.cmd'
      include 'com1f.cmd'
      include 'pigpen.cmd'
      include 'pigdung.cmd'
      include 'pufloc.cmd'
      include 'deffin.cmd'
      include 'latlon.cmd'
      include 'errlog.cmd'
      dimension hg(mmx,mmy,mmz)

      data pi /3.1415926/
      nskdmp = 0
c
      if (ifgnow.eq.0) then
        ix = ig
        jy = jg
        kz = kg
      else
        ix = ifn
        jy = jfn
        kz = kfn
      endif
         
c     idump = 0 means puff has not dumped
c     idump = 1 means puff is beyond edge of region, and is gone forever
c     idump = 2 means puff has dumped because it will enter a finer grid in 
c               next timestep
c     idump = 3 means some (or all) of puff has dumped because of cross-
c               sectional size 
c     idump = 4 means puff has dumped because of volume 
c     idump = 5 means puff is more than 24 hours old and is dumped
c
c-----check for idump = 1 (determined in ttraj); all necessary varible 
c     modifications have already been done

      if (idump(npuff).eq.1) goto 900

c-----check if idump = 4 (determined in expand); if so, prepare to dump entire 
c     puff

      if (idump(npuff).eq.4) then
        nskdmp = nskins(npuff)
        if (tpuff(npuff).lt.0.) then
          tpuff(npuff) = curtim + deltat - abs(tpuff(npuff))
          do 5 np = 1,nplm
            if (nplume(npuff).eq.npsrc(np)) tsrc(np) = -tsrc(np)
            do 6 nc = 1,nsrcmbn(npuff)
              if (isrcmbn(npuff,nc).eq.npsrc(np)) 
     &             tsrc(np) = -tsrc(np)
    6       continue
    5     continue
        endif
        write(*,*)'npuff # ',npuff,' has gotten too big'
        goto 140
      endif

c-----check to see if puff will be advected into a finer grid after next
c     time step; if puff volume exceeds fine gridcell volume, prepare to dump 
c     entire puff

      if (lcart) then
        jnxt = ifix((yloc(npuff) + deltat*v(ig,jg,kg))/deltay) + 1
        inxt = ifix((xloc(npuff) + deltat*u(ig,jg,kg))/deltax(jnxt)) + 1
      else
        jnxt = ifix(yloc(npuff) + deltat*v(ig,jg,kg)/deltay) + 1
        inxt = ifix(xloc(npuff) + deltat*u(ig,jg,kg)/deltax(jnxt)) + 1
      endif
      is = nskins(npuff) + 1
      a = rfacb(is)*sigma(npuff)/1000.
      b = rfacb(is)*sigmav(npuff)/1000.
      puffvol = xlnpuf(npuff)*pi*a*b
      do 10 nf = ifgnow+1,numfin
        if (ifgptr(nf).eq.ifgnow) then
          if ((inxt.eq.ixfb(nf).or.inxt.eq.ixfe(nf)).and.
     &       (jnxt.eq.jyfb(nf).or.jnxt.eq.jyfe(nf))) then
            if (lcart) then
              yy = yloc(npuff)-float(jyfb(nf)-1)*deltay+delfy(nf)
              jj = ifix(yy/delfy(nf)) + 1
              xx = xloc(npuff)-float(ixfb(nf)-1)*deltax(jg)
     &          +delfx(jj,nf)
              ii = ifix(xx/delfx(jj,nf)) + 1
            else
              yy = (yloc(npuff)-float(jyfb(nf)-1))*nhf(nf)+1.
              jj = ifix(yy) + 1
              xx = (xloc(npuff)-float(ixfb(nf)-1))*nhf(nf)+1.
              ii = ifix(xx) + 1
            endif
            delz = deltaz(nfx(nf),nfy(nf),mxfz,nfz(nf),
     &                    ii,jj,htf(1,1,nf),height(npuff))
            gridvol = (wdump/2.)*delz*delfy(nf)*delfx(jj,nf)
            if (puffvol.gt.gridvol) then
              idump(npuff) = 2
            endif
          endif
        endif
   10 continue

      if (idump(npuff).eq.2) then
        nskdmp = nskins(npuff)
        if (tpuff(npuff).lt.0.) then
          tpuff(npuff) = curtim + deltat - abs(tpuff(npuff))
          do 15 np = 1,nplm
            if (nplume(npuff).eq.npsrc(np)) tsrc(np) = -tsrc(np)
            do 16 nc = 1,nsrcmbn(npuff)
              if (isrcmbn(npuff,nc).eq.npsrc(np))
     &              tsrc(np) = -tsrc(np)
   16       continue
   15     continue
        endif
        write(*,*)'npuff # ',npuff,' going into next fine grid'
        goto 140
      endif

c-----loop over onion skins to check if puff will dump any

      if (kz.eq.1) then
        dpth = hg(ix,jy,kz)
      else
        dpth = hg(ix,jy,kz) - hg(ix,jy,kz-1)
      endif
      do 20 ns = 1,nskins(npuff)
        ea = pi*(sigma(npuff)*sigmav(npuff)/1.e6)*rfacb(ns+1)**2
        if (ea .gt. wdump*dy*dpth) then
          idump(npuff) = 3
          nskdmp = nskdmp + 1
        endif
   20 continue

      if (idump(npuff).eq.3) then 
        if (tpuff(npuff).lt.0.) then
          tpuff(npuff) = curtim + deltat - abs(tpuff(npuff))
          do 25 np = 1,nplm
            if (nplume(npuff).eq.npsrc(np)) tsrc(np) = -tsrc(np)
            do 26 nc = 1,nsrcmbn(npuff)
              if (isrcmbn(npuff,nc).eq.npsrc(np))
     &               tsrc(np) = -tsrc(np)
   26       continue
   25     continue
        endif
        write(*,*) 'npuff # ',npuff,' is dumping ',
     &              nskdmp,'skins because of cross-sectional area'
        write(*,*) '    grid ',ifgnow, '  cell ',ix,jy,kz
        goto 140
      endif

      age = (curtim - pufage(npuff))/60.
      if (age.gt.mxage) idump(npuff) = 5
c
      if (idump(npuff).eq.5) then 
        nskdmp = nskins(npuff)
        if (tpuff(npuff).lt.0.) then
          tpuff(npuff) = curtim + deltat - abs(tpuff(npuff))
          do 35 np = 1,nplm
            if (nplume(npuff).eq.npsrc(np)) tsrc(np) = -tsrc(np)
            do 36 nc = 1,nsrcmbn(npuff)
              if (isrcmbn(npuff,nc).eq.npsrc(np))
     &           tsrc(np) = -tsrc(np)
   36       continue
   35     continue
        endif
        write(*,*) 'npuff # ',npuff,' is dumping ',
     &               nskdmp,'skins because it is more than ',mxage,
     &               ' hours old'
        goto 140
      endif


c-----don't let puff dump if idump = 0
 
      if (idump(npuff).eq.0) goto 900
c
c******** set puff for dumping **********
c
  140 continue
      if (idiag .ne. 0) then
        write(*,2790) npuff
 2790   format('dumping puff # ',i3)
        write(nutrc,400) npuff,kpuff(npuff),nskdmp,ifgnow,ix,jy,kz
      endif
c
c     find next available index for dumping puff
c
      do 150 ii = 1, mxdmp
        if (tdmp(ii,1).le.0.) then
          iid = ii
          go to 160
        endif
  150 continue
      write(*,*) ' ran out of array space for dumping puffs.'
      write(8,*) ' ran out of array space for dumping puffs.'
      write(ierrlg,*) 1094,'  ran out of array space for dumping puffs.'
      stop4
c-----fill dump arrays with puff location, volume, effective time and
c     effective emissions rate.
 
  160 continue
      igrid = ifgnow + 1
      idmp(iid,igrid) = ix
      jdmp(iid,igrid) = jy
      kdmp(iid,igrid) = kz
      tdmp(iid,igrid) = tpuff(npuff)
      dtdmp(iid) = dtpuf(npuff)
      dchk = amax1(dtpuf(npuff),delpuf)
      if (tdmp(iid,igrid) .gt. 5.*dchk) then
        write(nutrc,2000) tdmp(iid,igrid) , delpuf
        tdmp(iid,igrid) = delpuf
      endif
c
      do 190 ns = 1,nskdmp
        if (ns.eq.1) voldmp(iid) = 0.
        ib = nskins(npuff)
        vdmp = pi*xlnpuf(npuff)*(sigma(npuff)*sigmav(npuff)/1.e6) *
     &         (rfacb(ib+1)**2 - rfacb(ib)**2)
        voldmp(iid) = voldmp(iid) + vdmp
        do 180 is = 1,nospec
          if (ns.eq.1) condmp(is,iid) = 0.
          condmp(is,iid) = condmp(is,iid) + 
     &               (concs(is+1,ib,npuff)-connul(is+1,ib,npuff))
     &                 *vdmp*1.e9/dtpuf(npuff)
c
          if (idiag .ne. 0) then
            write(*,2001) iid, ib, is, voldmp(iid),condmp(is,iid),
     &                   concs(is+1,ib,npuff)
 2001       format('dmp #= ',i4,' skin= ',i4,' spec= ',i4,' vol= ',
     &             e11.4,' mass= ',e11.4,' conc= ',e11.4)
          endif

  180   continue
        nskins(npuff) = nskins(npuff) - 1
  190 continue
      if (nskins(npuff).eq.0) npuffs = npuffs - 1
 
c-----load cell indices for parent grids (if any)

      if (ifgnow.gt.0) then
        inxgrd = ifgnow
  200   ifginr = inxgrd
        inxgrd = ifgptr(ifginr)

c-----coarse grid

        if (inxgrd.eq.0) then
          idmp(iid,1) = ig 
          jdmp(iid,1) = jg 
          kdmp(iid,1) = kg
          tdmp(iid,1) = tdmp(iid,igrid)
          if (idiag.ne.0) write(nutrc,400) npuff,kpuff(npuff),nskdmp, 
     &                    inxgrd,idmp(iid,1),jdmp(iid,1),kdmp(iid,1)
          goto 900

c-----fine grids

        else
          hrat = float(nhf(inxgrd))/float(nhf(ifgnow))
          idmp(iid,inxgrd+1) = nhf(inxgrd)*(ixfb(ifgnow)-ixfb(inxgrd))
     &        + int((float(ifn)-1.5)*hrat) + 2
          jdmp(iid,inxgrd+1) = nhf(inxgrd)*(jyfb(ifgnow)-jyfb(inxgrd))
     &        + int((float(jfn)-1.5)*hrat) + 2
          do 205 kkc1=1,nfz(inxgrd)
            if (kfn .le. kvmap(kkc1,ifginr)) then
c             kdmp(iid,inxgrd+1) = kvmap(kkc1,ifginr)
              kdmp(iid,inxgrd+1) = kkc1
              go to 210
            endif
  205     continue
  210     tdmp(iid,inxgrd+1) = tdmp(iid,igrid)
          if (idiag.ne.0) write(nutrc,400) npuff,kpuff(npuff),nskdmp, 
     &                      inxgrd,idmp(iid,inxgrd+1),
     &                      jdmp(iid,inxgrd+1),kdmp(iid,inxgrd+1)
          goto 200
        endif
      endif

  900 return
  400 format(' puff # ',i6,' count ',i6,' is dumping ',i2,
     &       ' skins into grid ',i2,', cell ',3i5)
 2000 format(1x,'**** oink! ****** warning ***** oink! ***********',/,
     &       1x,'******** but i mean serious warning *************',/,
     &       1x,'   bad pig dump time :  ',f10.4,' resetting to delpuf :
     &   ',  f10.4)
      end
