c***********************************************************************
      real function sigmaz (x,sigz0,is,iinit)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: sigmaz.f,v $
c  Revision 1.3  1996/06/14  23:04:12  tcm
c  miniumu sigma z value check added by gem
c
c  Revision 1.2  1994/02/10  21:00:23  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c
c
c
c
c         returns the vertical dispersion coefficient (sigma z)
c             gwl/sai  june 1988
c
c         uses seinfelds equations for the pasquill-gifford sigmaz
c         curves.  reference: atmospheric chemistry and physics of
c         air polution, john wiley and sons 1986, table 14.2, p577.
c
c         input parameters-
c             x      downwind distance in meters
c             sigz0  current sigmaz
c             is     stability class (1=a, 2=b,..., 6=f)
c             iinit  flag to indicate whether this call is to 
c                    initialize (=0 means initialize)
c
c         returns-
c             the vertical dispersion coefficient in meters
c
c         for iinit = 1, caculate alogx by reversing the equation
c         and then calculate sigmaz 
c 
c     -----------------------------------------------------------
c
      real x
      integer is
c
c     *** coefficients for sizgmz = exp(a+b*ln(x)+c*ln(x)**2 where
c         a, b, and c are differerent for each stability
      real a(6),b(6),c(6),szmax(6),szmin(6)
      data a/4.679,-1.999,-2.341,-3.186,-3.783,-4.490/
      data b/-1.7172,0.8752,0.9477,1.1737,1.3010,1.4024/
      data c/0.2770,0.0136,-0.0020,-0.0316,-0.0450,-0.0540/
      data szmax /2.46301e+09,19531.,4042.84,463.229,186.920,89.8521/
      data szmin /14.09,10.17,7.249,4.706,3.504,2.277/
c
c     --------------------------------------------------------------
c
      if (iinit .eq. 0) then
        alogx = alog(x)
        sigmaz = exp(a(is) + b(is)*alogx + c(is)*alogx*alogx)
c
      else if (iinit .eq. 1) then 
c
       if (sigz0 .lt. szmin(is)) then
	  xx=amax1(x,100.)
	  alogx=alog(xx)
          sigmaz = exp(a(is) + b(is)*alogx + c(is)*alogx*alogx)
c
c      if sigma is larger than maximum from formula for this
c      stability class, calculate an increment by tagging the
c      x increment onto 10**5
c
        elseif (sigz0 .gt. szmax(is)) then
          xx = 1.e5 + x
          alogx = alog(xx)
          signew = exp(a(is) + b(is)*alogx + c(is)*alogx*alogx)
          sigmaz = sigz0 + (signew - szmax(is))
        else
          alsig = alog(sigz0)
          alogx = (-b(is) + sqrt(b(is)**2 - 4.*c(is)*
     &           (a(is)-alsig)))/(2.*c(is))
          if (alogx .lt. 0.) stop53
          xx = exp(alogx) + x
          if (xx .lt. 1.) stop54
          alogx = alog(xx)
          sigmaz = exp(a(is) + b(is)*alogx + c(is)*alogx*alogx)
        endif
      else
        stop55
      endif
c
      if (sigmaz .lt. 1.) stop56
      if (iinit .eq. 1 .and. sigmaz .lt. sigz0) stop57
c
      return
      end
