      subroutine srlyrf(time,ndate,nxf,nyf, nfspec, ctlonf, ctlatf,
     &                  cloudf, temprf, tfsurf, uf, vf, presf,
     &                  htf, flandf, lstres, lsnow, ch2o, rain,
     &                  depvelf)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: srlyrf.f,v $
c  Revision 1.5  1999/12/21  19:28:31  tcm
c  Renamed com3 include file to comd3.
c
c  Revision 1.4  1999/05/12  16:47:32  dfg
c  Added kbsulf=ksulf because new chemistry solver routing uses ksulf.
c
c  Revision 1.3  1998/11/04  01:55:34  tcm
c  Revised date treatment.
c
c  Revision 1.2  94/02/10  21:00:39  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c  
c
c
c
c**********************************************************************
c
c-----this subroutine does the following tasks for a fine grid:
c
c     (1) estimation of the pbl parameters such as the monin-obukhov
c         length and ustar
c     (2) estimation of the dry deposition velocities at each grid cell
c         and for each species (gas and particle)
c
c-----this subroutine produces the following output:
c     depvelf - deposition velocities (m/s) for fine grid
c
c-----depvelf is landuse weighted for each grid cell
c
c-----this routine is called by:
c     aaamain
c
c-----this routine calls the following subroutines
c     (1) solar
c     (2) pblmod
c     (3) aerody
c     (4) bndry
c     (5) surfce
c     (6) prtdep
c
c-----this version of surlyr is by g moore may 1990
c     revised by c emery and t myers october 1991
c     srlyrf developed from surlyr by t myers, july 92
c-----enforced minimum wind speed of 0.01 m/s (tcm, 9/18/92)
c
c**********************************************************************
c
c*****introduces a number of comdeck inserts

      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'aerosl.cmd'
      include 'latlon.cmd'
      include 'comchem.cmd'
      include 'comd3.cmd'
c
c*****dimensions for arrays passed as arguments
c
      dimension ctlonf(nxf,nyf), ctlatf(nxf,nyf),
     &          cloudf(nxf,nyf), temprf(nxf,nyf), tfsurf(nxf,nyf), 
     &          uf(nxf,nyf), vf(nxf,nyf), presf(nxf,nyf),
     &          htf(nxf,nyf), flandf(nxf,nyf,mlus), lstres(nxf,nyf), 
     &          lsnow(nxf,nyf), ch2o(nxf,nyf), rain(nxf,nyf),
     &          depvelf(nxf,nyf,nfspec)

c
c*****local declarations and dimension statements

      dimension vmeanb(mlus), vmeang(mlus), vmeanr(mlus), vmean5(mlus)
      dimension ra(mlus), rs(mxspec,mlus), rb(mxspec,mlus)
      dimension drms1(nbin),drms2(nbin),drms3(nbin),drsz(nbin)
      real zz0(mlus), solflx(10), cldadj(11), cosz(10), wsrf(mlus)
      dimension flndcl(mlus)
c
c*****data statements for routine specific variables

      data rhop/1.5e+06/
      data zz0 /3., 0.25, 0.05, 1., 1., 1., 0.0001, 0.002, 
     &          0.15, 0.1, 0.1 /
      data solflx /915., 890., 840., 750., 630., 500., 345., 
     &          190., 60., 0./
      data cldadj /1., 0.91, 0.84, 0.79, 0.75, 0.72, 0.68, 
     &             0.62, 0.53, 0.41, 0.23/
      data cosz /1., 0.9962, 0.9659, 0.9063, 0.8192, 0.7071, 0.5736, 
     &           0.4226, 0.2588, 0.0872/
c
c
c In new chemistry solver, it used ksulf instead of kbsulf (dfg, 05/12/99)
c
      kbsulf=ksulf

c-----initialize some contants and arrays that are needed

      mbin = nbin
      do 33 l = 1,nbin
        drms1(l) = rmass1(l)
        drms2(l)=rmass2(l)
        drms3(l)=rmass3(l)
        drsz(l) = rsize(l)
   33 continue
      do 131 lu = 1,mlus
        ra(lu) = 0.
        do 133 l = 1,nfspec
          rs(l,lu) = 0.0
  133   continue
  131 continue
      do 134 i = 1,nxf
        do 136 j = 1,nyf
          do 135 l = 1,nfspec
            depvelf(i,j,l) = 0.
  135     continue
  136   continue
  134 continue
c
c-----convert date to day,month and year
c
      if (ndate .lt. 100000) then
        call jtocal(ndate,icdate)
        ny=icdate/10000
        nm=icdate/100-ny*100
        nd=icdate-nm*100-ny*10000
      else
        ny=ndate/10000
        nm=ndate/100-ny*100
        nd=ndate-nm*100-ny*10000
      endif
      if (ny .lt. 100) then
        if (ny .lt. 51) then
          ny = ny + 2000
        else
          ny = ny + 1900
        endif
      endif
c
c-----calculate the surface deposition rate
c-----row by row
c
      do 100 j=2,nyf-1
c
c-----calculate by each column element
c
        do 90 i=2,nxf-1

c-----calculate solar zenith angle and solar flux

          rllng = -ctlonf(i,j) 
          rllt = ctlatf(i,j)
          call solar(rllt,rllng,tz,ny,nm,nd,time,zcoef)
          ica = nint(10.*cloudf(i,j) + 0.5)
          do 95 ncz = 2,10
            if (zcoef.ge.cosz(ncz).and.zcoef.lt.cosz(ncz-1)) then
              icz = ncz-1
              goto 96
            endif
   95     continue
          icz = 10
   96     continue 
          gsun = cldadj(ica)*solflx(icz)
          tcell = temprf(i,j)
          wcell = amax1(0.01,16.67*sqrt(uf(i,j)**2 + vf(i,j)**2))
          pcell = 1013.*presf(i,j)
          tscell = tfsurf(i,j)
          topcell = 1.e3*htf(i,j)
         
c-----loop over non-zero land use categories for each cell to get
c     gaseous ra & rb, and particle deposition velocity

          do 50 lu = 1,mlus
            if (flandf(i,j,lu).lt.1.0e-3) goto 50
            z00 = zz0(lu)
c
c-----now estimates the monin-obukhov length and the ustar
c
            call pblmod(wcell,tcell,tscell,pcell,topcell,
     &                 z00,elinv,ustar,wref)
            wsrf(lu) = wref
            if (lu.eq.7) then
              z00 = 1.6e-6/ustar + 1.6e-3*ustar**2
            endif
c
c-----now estimate the aerodynamic resistance, and neutral and 
c     over-water ustar for further use
c
            call aerody(lu,ustar,elinv,z00,ra)
            ustrn = sqrt(wsrf(lu)/ra(lu))
            if (lu.eq.7) ustrw = ustar
c
c-----now estimate the laminar flow resistance
c
            call bndry(ustrn,rb(1,lu),nfspec)
c
c------now estimate the deposition velocity of particles for
c------so4 and hso5 - assume particle density is 1.5e+06 g/m3

            vmeanb(lu) = 0.0
            vmeang(lu) = 0.0
            vmeanr(lu) = 0.0
            vmean5(lu) = 0.0
            if (kbsulf.ne.0)
     &         call prtdep(drsz,drms2,mbin,ustrn,
     &                    z00,rhop,tcell,ra(lu),vmeanb(lu))
            if (kgsulf.ne.0)
     &         call prtdep(drsz,drms3,mbin,ustrn,
     &                    z00,rhop,tcell,ra(lu),vmeang(lu))
            if (krsulf.ne.0)
     &         call prtdep(drsz,drms1,mbin,ustrn,
     &                   z00,rhop,tcell,ra(lu),vmeanr(lu))
            if (khs5n.ne.0) 
     &         call prtdep(drsz,drms1,mbin,ustrn,
     &                     z00,rhop,tcell,ra(lu),vmean5(lu))

   50     continue
c
c-----now estimate the canopy (surface) resistance
c
          do 52 lu=1,mlus
            flndcl(lu) = flandf(i,j,lu)
   52     continue
          call surfce(nfspec,nm,gsun,ustrw,wsrf,tcell,pcell,
     &                lstres(i,j),lsnow(i,j),ch2o(i,j),
     &                flndcl,rain(i,j),rs)
c
c------estimates the deposition velocity (m/s)

          do 85 k=1,nfspec
            do 55 lu = 1,mlus
              if (flandf(i,j,lu).lt.1.e-3) goto 55
              if (k .eq. kbsulf) then
                vdep = vmeanb(lu)
              elseif (k .eq. kgsulf) then
                vdep = vmeang(lu)
              elseif (k .eq. krsulf) then
                vdep = vmeanr(lu)
              elseif (k .eq. khs5n) then
                vdep = vmean5(lu)
              else
                vdep=1.0/(ra(lu) + rb(k,lu) + rs(k,lu))
              endif
 
c-----running sum over landuse becomes landuse-weighted deposition velocity
c     for species k

              depvelf(i,j,k) = depvelf(i,j,k) + flandf(i,j,lu)*vdep

   55       continue

c-----the total landuse-weighted deposition velocity (m/s)

            depvelf(i,j,k) = -depvelf(i,j,k)

   85     continue

   90   continue
  100 continue
c
      return
      end
