      subroutine surfce(nospec,nm,gsun,ustrw,wsrf,tcellk,pcell,lstrcl,
     &                  lsnocl,h2ocl,flndcl,raincl,rs)
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: surfce.f,v $
c  Revision 1.7  1999/12/21  19:28:40  tcm
c  Renamed com3 include file to comd3.
c
c  Revision 1.6  1999/06/01  16:20:42  dfg
c  Added lmap for ALDX (same as ACET) and PANX (same as PAN).
c  Actived lmap for MEOH due to TOX chemistry.
c
c  Revision 1.5  1999/05/14  17:09:46  dfg
c  added more values (0's) to lmap initialization because the
c  dimension is 40 now.
c
c  Revision 1.4  1999/05/13  17:33:02  dfg
c  Rename dratio to dratio1 so that it would not mix up with
c  dratio defined in comchem for used in BNDRY routine.
c
c  Revision 1.3  1996/06/14  23:07:22  tcm
c  see comments below
c
c  Revision 1.2  1994/02/10  21:01:34  tcm
c  Cosmetic revisions to code by gem, syw, tcm
*
* added ispd due to new Carter isop mechanism. set similar to form
* increased number od species from 30 to 32 to do this, and hence changed
* data statements here.
* 1996/06/07 hpd
*
* moved rcl and rgs calculation selon tcm, 1996/06/10, hpd
c
c
c**********************************************************************
c
c  begin new surface resistance section -- mepl 5-90
c
c     surface layer resistance ****************************************
c
c     calculated as a function of season, stabilty and
c     land use type
c
c     gaseous depositing species are assumed to be soluble and/or reactive
c     note that the indeces here are not the same as for the concentrations
c
c       kt(1)   so2
c       kt(2)   o3
c       kt(3)   h2o2
c       kt(4)   hno3
c       kt(5)   hno2
c       kt(6)   no2
c       kt(7)   no
c       kt(8)   co2     * (not a state species)
c       kt(9)   nh3     *
c       kt(10)  pan
c       kt(11)  hcho
c       kt(12)  hcooh
c       kt(13)  hcl     *
c       kt(14)  mhp
c       kt(15)  paa
c       kt(16)  oh      *
c       kt(17)  ho2     *
c       kt(18)  hno4
c       kt(19)  mgly
c       kt(20)  cres
c       kt(21)  meoh
c       kt(22)  acet
c       kt(23)  open
c  also,kn2o5   n2o5
c
c     from m.l. wesely, atmos environ 23(6):1293-1304 (1989)
c
c-----the input to this subroutine is the following:
c-----nospec = number of simulated species
c-----nm = month
c-----gsun = solar insolation (w/m2) at ground
c-----ustrw = friction velocity of water
c-----wsrf = 10 m wind speed (m/sec)
c-----tcellk = ambient cell temperature (k)
c-----pcell = ambient cell pressure (mb)
c-----lstrcl = stress code for this cell
c-----lsnocl = snow code for this cell
c-----h2ocl = water vapor concentration for this cell (ppm-v)
c-----flndcl = fractional land use values for this cell
c-----raincl = rainfall rate this cell (inches/hour)
c
c-----the output from this subroutine is the following:
c-----rs = the array of surface resistances (species dependent)
c
c-----this subroutine was restructured by gem on may 1990
c
c-----revised according to review by sigma and restructured
c     to improve efficiency.  (tcm, 9/91)
c
c-----moved some parameters to argument list to avoid needing
c     to include common blocks and pass location. (tcm, 7/92)
c
c**********************************************************************
c
c******   inserts the parameter file
       include 'parm.cmd'
      include 'parchm.cmd'
c
c******   inserts the common blocks
c
c    comchem,&com3 are included in bd1
      include 'comchem.cmd'
      include 'comd3.cmd'
c
c 
c******   local arrays
      dimension rmin(mlus,5),rcuts(mlus,5),rcls(mlus,5),rgss(mlus,5),
     &rac(mlus,5), rsurf(mlus,nhen),rstom(mlus,nhen),rmes(nhen),
     &rcut(mlus,nhen), rcl(mlus,nhen),rgs(mlus,nhen),
     &rcutwet(mlus,nhen),rstrf(mlus,nhen),
     &dratio1(nhen),freact(nhen),rclo(mlus,5),rgso(mlus,5)
      dimension rs(mxspec,mlus), wsrf(mlus)
      dimension tmax(mlus), tmin(mlus), topt(mlus)
      dimension lmap(mxspec)
      dimension flndcl(mlus)
      logical ldew, lfirst
      save lfirst, lmap
c
      data lfirst /.true./
      data lmap /0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
     &           0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0/
c
c  dratio1(i) = diffusivity(h2o)/diffusivity(i)
c  calculated as the sqrt of the ratio of the molecular weights: mw(i)/mw(h2o)
c
      data dratio1/1.9,1.6,1.4,1.9,1.6,1.6,1.3,1.6,1.0,2.6,1.3,1.6,
     &  1.4,1.6,2.0,1.0,1.4,2.1,2.0,2.4,1.3,1.6,2.5/
c
c  freact is a reactivity parameter
c
      data freact/0.,1.,1.,0.,0.1,0.1,0.,0.,0.,0.1,0.,0.,0.,0.3,0.5,
     &  1.,0.1,0.,0.,0.,0.,0.,0./
c
c  the following resistances are from wesely (1989)
c
      data rmin/
     & 99999,60.  ,120. ,70.  ,130,100,99999,99999,80.  ,100. ,150. ,
     & 99999,99999,99999,99999,250,500,99999,99999,99999,99999,99999,
     & 99999,99999,99999,99999,250,500,99999,99999,99999,99999,99999,
     & 99999,99999,99999,99999,400,800,99999,99999,99999,99999,99999,
     & 99999,120. ,240. ,140. ,250,190,99999,99999,160. ,200. ,300. /
c
      data rcuts/
     & 99999,2000.,2000.,2000.,2000,2000.,99999,99999,2500,2000,4000,
     & 99999,9000.,9000.,9000.,4000,8000.,99999,99999,9000,9000,9000,
     & 99999,99999,9000.,9000.,4000,8000.,99999,99999,9000,9000,9000,
     & 99999,99999,99999,99999,6000,9000.,99999,99999,9000,9000,9000,
     & 99999,4000.,4000.,4000.,2000,3000.,99999,99999,4000,4000,8000/
c
      data rac/
     & 100,200,100,2000,2000,2000,0.5,0.5,300,150,200,
     & 100,150,100,1500,2000,1700,0.5,0.5,200,120,140,
     & 100,10.,100,1000,2000,1500,0.5,0.5,100,50.,120,
     & 100,10.,10.,1000,2000,1500,0.5,0.5,50.,10.,50.,
     5 100,50.,80.,1200,2000,1500,0.5,0.5,200,60,120./
c
      data rgss/
     & 400,150,350,500,500,100,0.5,1000,0.5,220,400,
     & 400,200,350,500,500,100,0.5,1000,0.5,300,400,
     & 400,150,350,500,500,200,0.5,1000,0.5,200,400,
     & 100,100,100,100,100,100,0.5,1000,100,100,50.,
     & 500,150,350,500,500,200,0.5,1000,0.5,250,400/
c
      data rgso/
     & 300,150.,200.,200.,200.,300.,2000,400,1000,180.,200.,
     & 300,150.,200.,200.,200.,300.,2000,400,800.,180.,200.,
     & 300,150.,200.,200.,200.,300.,2000,400,1000,180.,200.,
     & 600,3500,3500,3500,3500,3500,2000,400,3500,3500,3500,
     & 300,150.,200.,200.,200.,300.,2000,400,1000,180.,200./
c
      data rcls/
     & 99999,2000.,2000.,2000,2000,2000,99999,99999,2500,2000.,4000,
     & 99999,9000.,9000.,9000,2000,4000,99999,99999,9000,9000.,9000,
     & 99999,99999,9000.,9000,3000,6000,99999,99999,9000,9000.,9000,
     & 99999,99999,99999,9000,200.,400.,99999,99999,9000,99999,9000,
     & 99999,4000.,4000.,4000,2000,3000,99999,99999,4000,4000.,8000/
c
      data rclo/
     & 99999,1000,1000,1000,1000,1000,99999,99999,1000,1000,1000,
     & 99999,400.,400.,400.,1000,600.,99999,99999,400.,400.,400.,
     & 99999,1000,400.,400.,1000,600.,99999,99999,800.,600.,600.,
     & 99999,1000,1000,400.,1500,600.,99999,99999,800.,1000,800.,
     & 99999,1000,500.,500.,1500,700.,99999,99999,600.,800.,800./
c
      data tmax /40.,40.,40.,40.,40.,40.,40.,40.,40.,40.,40./
      data tmin / 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0./
      data topt /20.,20.,20.,20.,20.,20.,20.,20.,20.,20.,20./
 
c
c   first time through only, set up species mapping
c
      if (lfirst) then
        do 25 l=1,nospec
          if (kno .eq. l) lmap(l) = 7
          if (kno2 .eq. l) lmap(l) = 6
          if (ko3 .eq. l) lmap(l) = 2
          if (khcho .eq. l) lmap(l) = 11
          if (kacet .eq. l) lmap(l) = 22
          if (kaldx .eq. l) lmap(l) = 22
          if (kcres .eq. l) lmap(l) = 20 
          if (kmgly .eq. l) lmap(l) = 19
          if (kopen .eq. l) lmap(l) = 23
          if (kpna .eq. l) lmap(l) = 18
c         if (knxoy .eq. l) lmap(l) = 4
          if (kpan .eq. l) lmap(l) = 10
          if (kpanx .eq. l) lmap(l) = 10
          if (khno2 .eq. l) lmap(l) = 5
          if (kh2o2 .eq. l) lmap(l) = 3
          if (khno3 .eq. l) lmap(l) = 4
          if (ksiv .eq. l) lmap(l) = 1
          if (kmeoh .eq. l) lmap(l) = 21
          if (kispd .eq.l) lmap(l) = 11
   25   continue
        lfirst = .false.
      endif
c
c    ** need to determine iseasn from month and snow cover **
c       iseasn = 1  summer
c                2  early autumn
c                3  late autumn after frost
c                4  winter with snow on ground
c                5  spring 
c 
       if(nm .ge. 11)then 
         iseasn=3
       elseif(nm .eq. 10)then
         iseasn=2
       elseif(nm.ge.6)then
         iseasn=1
       elseif(nm.ge.3)then
         iseasn=5
       else
         iseasn=3
       endif
       if(lsnocl.gt.0) iseasn = 4
c
c  call tdrate to get henry's law constants at the surface temp
c  ** still need to adjust for pressure!! **
c
      tcell = tcellk - 273.0
      call tdrate(nreact,tcellk,pcell)
c
c   calculate relative humidity
c
       ldew = .false.
       call vapsrc(tcellk,eqvap)
       rhum = h2ocl/eqvap*1.e-4
c
c
c  calculate solar correction factor for stomatal resistance
c
      gofs = 1. + (200./(gsun+0.1))**2
c
      rdc=100.*(1.+1000./(gsun+10.))
      heffso = hheff(1)
      do 205 lu = 1,mlus
c
c      calculate temperature correction factor for stomatal resistance
c
        if (tcell .lt. tmin(lu)+.01 .or.
     &      tcell .gt. tmax(lu)-.01) then
          goft = 1.e4
        else
          tmxmn = tmax(lu) - tmin(lu)
          tmxo = tmax(lu) - topt(lu)
          tomn = topt(lu) - tmin(lu)
          goft = (tomn/(tcell - tmin(lu)))
     &           * (tmxo/(tmax(lu) - tcell))**(tmxo/tmxmn)
        endif
c
        do 200 l = 1,nhen
          rcut(lu,l)=rcuts(lu,iseasn)/(hheff(l)/heffso + freact(l))
c
c       for now, stressed vegetation is set to 10 times stomatal
c       resistance otherwise calculated.  this may not be the 
c       most appropriate multiplier.
c
          if (lstrcl .eq. 1) then
            rstrf(lu,l)=dratio1(l)*rmin(lu,iseasn)*gofs*goft*10.
          else
            rstom(lu,l)=dratio1(l)*rmin(lu,iseasn)*gofs*goft
          endif
  200   continue
c
  205 continue
c
      do 280 l=1,nhen
        rmes(l) = 1./(hheff(l)/3000. + 100.*freact(l))
        do 270 lu=1,mlus
          if(flndcl(lu) .lt. 1.0e-3) then
            rsurf(lu,l) = 1.e10
            go to 270
          endif
c
c   check for presence of dew
c
          rtest = (100. - rhum) * (wsrf(lu)+0.6)
          if (rtest .lt. 19.) ldew = .true.
c
c     note that the order of the following checks is important.
c     that is, water surface takes priority over rain wetted which
c     takes priority over dew wetted which takes priority over 
c     regular surface.
c
          rcl(lu,l)=1./(hheff(l)/(heffso*rcls(lu,iseasn))+freact(l)/
     &                rclo(lu,iseasn))
          rgs(lu,l)=1./(hheff(l)/(heffso*rgss(lu,iseasn))+freact(l)/
     &                rgso(lu,iseasn))
c
c        water surface
c
          if (lu .eq. 7) then
            rsurf(lu,l) = 1./(hheff(l)*0.082*tcellk*4.8e-4*ustrw)
c
c        rain wetted surface
c
          else if (raincl .ge. 1.) then
            if (l .eq. 1) then
              if (lu .eq. 1) then
                rcutwet(lu,l) = 50.
              else
                rcutwet(lu,l) = 1./(1./5000. + 1./(3.*rcut(lu,l)))
              endif
            else if (l .eq. 2) then
              rcutwet(lu,l) = 1./(1./1000. + 1./(3.*rcut(lu,l)))
            else
              rcutwet(lu,l) = 1./(1./(3.*rcut(lu,l))+1e-7*hheff(l) +
     &                    freact(l)/rcutwet(lu,2))
            endif
            rsurf(lu,l)=1./(1./(3.*rstom(lu,l)+rmes(l))+1/rcutwet(lu,l)
     &                +1/(rdc+rcl(lu,l))+1/(rac(lu,iseasn)+rgs(lu,l)))
c
c        dew wetted surface
c
          else if (ldew) then
            if (l .eq. 1) then
              if (lu .eq. 1) then
                rcutwet(lu,l) = 50.
              else
                rcutwet(lu,l) = 100.
              endif
            else if (l .eq. 2) then
              rcutwet(lu,l) = 1./(1./3000. + 1./(3.*rcut(lu,l)))
            else
              rcutwet(lu,l) = 1./(1./(3.*rcut(lu,l))+1e-7*hheff(l) +
     &                    freact(l)/rcutwet(lu,2))
            endif
            rsurf(lu,l)=1./(1./(3.*rstom(lu,l)+rmes(l))+1/rcutwet(lu,l)
     &                +1/(rdc+rcl(lu,l))+1/(rac(lu,iseasn)+rgs(lu,l)))
c
c        regular dry surface
c
          else
            if (lstrcl .le. 0) then
              rsurf(lu,l)=1./(1./(rstom(lu,l)+rmes(l))+1./rcut(lu,l)+
     &           1./(rdc+rcl(lu,l))+1./(rac(lu,iseasn)+rgs(lu,l)))
            else if (lstrcl .eq. 1) then
              rsurf(lu,l)=1./(1./(rstrf(lu,l)+rmes(l))+1./rcut(lu,l)+
     &           1./(rdc+rcl(lu,l))+1./(rac(lu,iseasn)+rgs(lu,l)))
            else if (lstrcl .ge. 2) then
              rsurf(lu,l)=1./(1./(10000.+rmes(l))+1./rcut(lu,l)+
     &           1./(rdc+rcl(lu,l))+1./(rac(lu,iseasn)+rgs(lu,l)))
            endif
          endif
  270   continue
  280 continue
c
c
      do 305 n=1,mlus
        do 301 l=1,nospec
c
c  set rsurface very high for non-depositing species
c
          rs(l,n)=1.0e+10
c
          ld = lmap(l)
          if (ld .gt. 0) rs(l,n) = rsurf(n,ld)
  301   continue
  305 continue
c
      return
      end
