      subroutine ttraj(deltat,time,idate,idebug) 
c
c     *** (c) Copr. 1992  Systems Applications, Inc.  San Rafael, CA
c
c$Log: ttraj.f,v $
c  Revision 1.13  1999/12/21  21:10:38  tcm
c  Cleaned up some format statements.
c
c  Revision 1.11  1999/03/20  01:38:37  tcm
c  Added recalculation of fine grid vertical index after location update.
c  (New routine fkindx is called.)
c
c  Revision 1.10  1998/11/19  00:11:50  ara
c  Eliminated error message directed to diag file (8)
c  as it was already directed by variable nutrc
c  Also eliminated error message directed to unit 9
c
c  Revision 1.9  1998/11/16  21:27:29  ara
c  Directed error message to screen, diag and log file (ierrlg)
c
c  Revision 1.8  1998/11/13  19:11:57  ara
c  Included errlog.cmd file to introduce ierrlg variable
c  Directed error message to diag and log file (ierrlg)
c
c  Revision 1.7  1998/11/04  02:04:00  tcm
c  Made diagnostic of chemistry calls active only when idiag .ne. 0.
c  /.
c
c  Revision 1.6  1996/06/14  23:07:52  tcm
c  see comments below
c
c  Revision 1.5  1995/04/26  00:46:16  tcm
c  	added code to correctly handle plume locations when in lat/lon coords
c
c  Revision 1.4  1995/04/10  22:17:30  tcm
c  Moved abs(nskins()) to inside of 600 loop to avoid erroneous number
c  of skins in puffdmp.
c
c  Revision 1.3  94/05/16  15:04:29  tcm
c  Plume expansion revised by CAE.
c  Initial plume expansion calculated and F stability eliminated.
c  Additional optional diagnostics added.
c  
c  Revision 1.2  94/02/10  21:02:12  tcm
c  Cosmetic revisions to code by gem, syw, tcm
c  
c  Added checks of puff dimensions and dilution when new puffs are 
c  initialized to avoid problems with calm winds and/or excessive
c  NOx concentrations in the puffs.     96/02/20  gem
c
c  Added loop 3111 to reset idump for new puffs
c  --- 96/03/27 gem
c
c-----driver for the uam-v plume-in-grid (pig) treatment
c     - based on paris
c     - evaporating onion skin configuration
c     - all time quantities are in minutes; curtim is minutes from midnight 
c       on the first day of the simulation.
c
c     from input:  wdump, nrclls, delpuf, xcmbn, delout
c
c     t myers,   c emery    11/91
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'comd3.cmd'
      include 'deffin.cmd'
      include 'latlon.cmd'
      include 'pigpen.cmd'
      include 'pufloc.cmd'
      include 'pigdung.cmd'
      include 'rpm.cmd'
      include 'com1f.cmd'
      include 'stack.cmd'
      include 'efcon.cmd'
      include 'balanc.cmd'
      include 'pigdiag.cmd'
      include 'errlog.cmd'
      real out1(mxskn),out2(mxskn),out3(mxskn),out4(mxskn)
      data pi /3.1415927/
      data icnt /0/
      save icnt
c
c
c  ------------------------------------------------------------
c
c  bit fourteen of idebug on (i.e., the 8192s location) will turn
c  on debug writes in ttraj.
c
      idiag = and(idebug,8192)
      if (idiag.ne.0) write(*,*) 'entering ttraj',time,idate
      lerror = .false.
      npufmx = mxpfs
      dumtim = aint(time)
      if (amod(dumtim,100.).eq.0. .and. idiag .ne. 0) then
         write(*,*)
         write(*,*)
         write(*,*)'For hour interval',dumtim-100.,' to',dumtim
         write(*,*)'Number of total puff chemistry calls: ',nchem/2
         write(*,*)'Number of times NOx > 10 ppm: ',nskip
         if (nchem .gt. 0) 
     &       write(*,*)'Percent of chemistry calls NOx > 10 ppm: ',
     &              100.*float(nskip)/float(nchem/2)
         write(*,*)
         write(*,*)
         nchem = 0
         nskip = 0
	 if ( icnt .gt. 0 ) then
	    write(*,*) '***** WARNING *****'
           write(*,*) 'Plume dimensions modified for at least one puff',
     &                 ' at initialization. '
	    write(*,*) 'Plume dimensions modified due to excessive NOx',
     &             ' emissions and/or insufficient ( <100X ) dilution.'
	    icnt = 0
	 endif
      endif
c
c  loop over puffs
c
      if (idiag.ne.0) write(*,*) 'beginning puff loop; curtim = ', 
     &   curtim
      do 200 n = 1, npufmx
        if (nskins(n) .eq. 0) go to 200
        if (idiag.ne.0.and.n.eq.1) write(*,*) 'moving puff # ',n,
     &      ' nskins = ',nskins(n)
c
c  load local puff location variables
c
        ifgnow = ifg(n)
        ic = iic(n)
        jc = jjc(n)
        kc = kkc(n)
        icold = ic
        jcold = jc
        kcold = kc
        xc = xloc(n)
        yc = yloc(n)
        if (ifgnow.ne.0) then
          if = iif(n)
          jf = jjf(n)
          kf = kkf(n)
          if (lcart) then 
            yf = yc-float(jyfb(ifgnow)-1)*deltay+delfy(ifgnow)
            xf = xc-float(ixfb(ifgnow)-1)*deltax(jc)+delfx(jf,ifgnow)
          else
            yf = (yc-float(jyfb(ifgnow)-1))*float(nhf(ifgnow))+1.
            xf = (xc-float(ixfb(ifgnow)-1))*float(nhf(ifgnow))+1.
          endif
        endif
        if (idiag.ne.0.and.n.eq.1) then
          write(*,*)'ifgnow,ic,jc,kc,xc,yc,if,jf,kf'
          write(*,*)ifgnow,ic,jc,kc,xc,yc,if,jf,kf
        endif
c
c  initialize variables for calculating trajectory step
c
        dtrem = deltat
        dttot = 0.
        dstot = 0.
c
c  trjstp takes a step that reaches the boundary of the finest grid cell 
c  (top, bottom or one of the sides) that the current puff occupies.  keep 
c  taking steps with trjstp until one coarse grid step is completed.
c
   10   ifgold = ifgnow
        if (ifgnow .eq. 0) then
          if (lcart) then 
            xxc = xc
            yyc = yc
          else
            xxc = xc*deltax(jc)
            yyc = yc*deltay
          endif
          call trjstp(mxx,mxy,mxz,nox,noy,nlayer,u,v,h,deltax(jc),
     &                 deltay,xxc,yyc,height(n),dtrem,dtc,dels,
     &                 ucomp,vcomp,ic,jc,kc,idiag)
          if (lcart) then 
            xc = xxc
            yc = yyc
          else
            xc = xxc/deltax(jc)
            yc = yyc/deltay
          endif
        else
          if (lcart) then 
            xxf = xf
            yyf = yf
          else
            xxf = xf*delfx(jf,ifgnow)
            yyf = yf*delfy(ifgnow)
          endif
          call trjstp(nfx(ifgnow),nfy(ifgnow),mxfz,nfx(ifgnow),
     &                nfy(ifgnow),nfz(ifgnow),uf(1,1,ifgnow),
     &                vf(1,1,ifgnow),htf(1,1,ifgnow),
     &                delfx(jf,ifgnow),delfy(ifgnow),xxf,yyf,
     &                height(n),dtrem,dtc,dels,ucomp,vcomp,
     &                if,jf,kf,idiag)
          if (lcart) then 
            xf = xxf
            yf = yyf
          else
            xf = xxf/delfx(jf,ifgnow)
            yf = yyf/delfy(ifgnow)
          endif
c
c  find new coarse grid location
c
          if (lcart) then
            xc = xf-delfx(jf,ifgnow)+float(ixfb(ifgnow)-1)*deltax(jc)
            yc = yf-delfy(ifgnow)+float(jyfb(ifgnow)-1)*deltay
            jc = ifix(yc/deltay) + 1
            ic = ifix(xc/deltax(jc)) + 1
          else
            xc = (xf-1.)/float(nhf(ifgnow))+float(ixfb(ifgnow)-1)
            yc = (yf-1.)/float(nhf(ifgnow))+float(jyfb(ifgnow)-1)
            jc = ifix(yc) + 1
            ic = ifix(xc) + 1
          endif
          do 20 k = 1,nlayer
            if (height(n).lt.h(ic,jc,k)) then
              kc = k
              goto 21
            endif
   20     continue
          kc = mxz + 100
   21     continue
        endif
c
c  is puff outside coarse grid domain?
c
        if (istr(jc).gt.nox .or. ic.le.istr(jc) .or. ic.ge.iend(jc)
     &      .or. kc.gt.nlayer) then
          if (tpuff(n) .gt. 0.) then
            tdel = tpuff(n)
          else
            tdel = curtim + deltat - abs(tpuff(n))
          endif
          do 26 ib = 1,nskins(n)
            do 25 is = 1,nospec
              vdmp = pi*1.e3*xlnpuf(n)*sigma(n)*sigmav(n) *
     &              (rfacb(ib+1)**2 - rfacb(ib)**2)
              pxflow(is) = pxflow(is) + (concs(is+1,ib,n) -
     &                     connul(is+1,ib,n))*1.e-6*vdmp
     &                     * tdel/deltat
   25       continue
   26     continue
          idump(n) = 1
          nskins(n) = 0
          npuffs = npuffs - 1
c
c  following needs to be done only when puff has none behind it
c
          if (tpuff(n) .lt. 0.) then
            do 30 np = 1,nplm
              if (nplume(n).eq.npsrc(np)) tsrc(np) = -tsrc(np)
              do 36 nc = 1,nsrcmbn(n)
                if (isrcmbn(n,nc).eq.npsrc(np))
     &                tsrc(np) = -tsrc(np)
   36         continue
   30       continue
          endif
          write(*,*) 'puff # ',n,' out of domain'
c         write(9,*) 'puff # ',n,' out of domain'
          go to 200
        endif
c
c  determine which fine grid (if any) puff is located in
c
        ifgnow = 0
        do 40 ifine=1,numfin
          if (ic .ge. ixfb(ifine) .and. ic .le. ixfe(ifine) .and.
     &       jc .ge. jyfb(ifine) .and. jc .le. jyfe(ifine)) then
             ifgnow = ifine
          endif
   40   continue
c
c  if puff is in a new fine grid, find new fine grid locations
c
        if (ifgnow.ne.ifgold.and.ifgnow.gt.0) then
          if (ifgnow.gt.ifgold) then
            write(8,*)'puff #',n,' entering a finer grid'
          endif
          if (lcart) then
            yf = yc-float(jyfb(ifgnow)-1)*deltay+delfy(ifgnow)
            jf = ifix(yf/delfy(ifgnow)) + 1
            xf = xc-float(ixfb(ifgnow)-1)*deltax(jc)+delfx(jf,ifgnow)
            if = ifix(xf/delfx(jf,ifgnow)) + 1
          else
            yf = (yc-float(jyfb(ifgnow)-1))*float(nhf(ifgnow))+1.
            jf = ifix(yf) + 1
            xf = (xc-float(ixfb(ifgnow)-1))*float(nhf(ifgnow))+1.
            if = ifix(xf) + 1
          endif
        endif
c
c   update new fine grid layer indices for puff
c
        if ( ifgnow .gt. 0 ) then
          call fkindx(nfx(ifgnow),nfy(ifgnow),mxfz,
     &                nfx(ifgnow),nfy(ifgnow),nfz(ifgnow),
     &                htf(1,1,ifgnow),height(n),
     &                if,jf,kf,idiag)
        endif
c
c  update trajectory length, remaining time and total time
c  loop back to 10 to finish coarse grid timestep
c
        dstot = dstot + dels
        dttot = dttot + dtc
        dtrem = deltat - dttot
        if (dtrem .gt. .01) go to 10
        if (idiag.ne.0.and.n.eq.1) then
          write(*,*) 'n,xc,yc,height,ic,jc,kc,ifgnow,if,jf,kf'
          write(*,*)  n,xc,yc,height(n),ic,jc,kc,ifgnow,if,jf,kf 
        endif
c
c  load puff-dependent location variables
c
        ifg(n) = ifgnow
        iic(n) = ic
        jjc(n) = jc
        kkc(n) = kc
        iif(n) = if
        jjf(n) = jf
        kkf(n) = kf
        xloc(n) = xc
        yloc(n) = yc
c
c  expand puff via stability class
c
        if (ifgnow .eq. 0) then
          call expand(mxx,mxy,mxz,nlayer,ic,jc,kc,u,v,temper,
     &                  tsurf(ic,jc),press,h,kv,height(n),sigma(n),
     &                  sigmav(n),rfacb(nskins(n)+1),xlnpuf(n),dstot,
     &                  wdump,deltax(jc),deltay,idump(n),0,idiag,
     &                  dum1,dum2,dum3,n)
        else
          call expandf(nfx(ifgnow),nfy(ifgnow),mxfz,nfz(ifgnow),
     &                  if,jf,kf,uf(1,1,ifgnow),vf(1,1,ifgnow),
     &                  temprf(1,1,ifgnow),tsurf(ic,jc),
     &                  presf(1,1,ifgnow),htf(1,1,ifgnow),
     &                  fkv(1,1,ifgnow),height(n),sigma(n),
     &                  sigmav(n),rfacb(nskins(n)+1),xlnpuf(n),dstot,
     &                  wdump,delfx(jf,ifgnow),delfy(ifgnow),idump(n),
     &                  0,idiag,dum1,dum2,dum3,n)
          do 51 k = 1,nlayer
            if (height(n).lt.h(ic,jc,k)) then
              kc = k
              goto 52
            endif
   51     continue
   52     continue
        endif
c
c  set up some parameters for rpm chemistry
c
        if (ifgnow.eq.0) then
           call setprm(mxx,mxy,mxz,mxspec,n,ic,jc,kc,nospec,
     &                  temper,conc,deltat,deltax(jc),deltay)
        else
           call setprm(nfx(ifgnow),nfy(ifgnow),nfz(ifgnow),nospec,
     &                 n,if,jf,kf,nospec,
     &                 temprf(1,1,ifgnow),cncf(1,1,1,ifgnow),
     &                 deltat,delfx(jf,ifgnow),delfy(ifgnow))
        endif
  200 continue
c
c  call rpm
c
      if (idiag.ne.0) write(*,*) '1st rpm call'
      call rpm(nutrc,nospec,deltat,time,idate,1,lerror)
      if (lerror) go to 800
c
c     call rpm for zero emissions (background) puffs
c
      if (idiag.ne.0) write(*,*) '2nd rpm call'
      call rpm(nutrc,nospec,deltat,time,idate,0,lerror)
      if (lerror) go to 800
c
c  determine if a portion (or all) of the puff should be dumped into the grid
c
      do 300 n = 1,npufmx
        if (nskins(n).eq.0) goto 300
        if (ifg(n).eq.0) then
          call puffdmp(mxx,mxy,mxz,n,ifg(n),iic(n),jjc(n),kkc(n),
     &            iic(n),jjc(n),kkc(n),h,deltay,deltat,idiag)
        else
          call puffdmp(nfx(ifg(n)),nfy(ifg(n)),mxfz,n,ifg(n),
     &            iic(n),jjc(n),kkc(n),iif(n),jjf(n),kkf(n),
     &            htf(1,1,ifg(n)),delfy(ifg(n)),deltat,idiag)
        endif
  300 continue
c
c--------------------------------------------------------------------------
c
c   write out current puff data
c
      if (curtim .ge. outtim - .001) then
        write(8,351) npuffs, curtim
        write(8,350)
        do 310 n = 1, npufmx
          if (nskins(n).gt.0) then
            write(8,360) n, kpuff(n), nplume(n),xloc(n), yloc(n),
     &           height(n), sigma(n), sigmav(n), idump(n), tpuff(n),
     &           dtpuf(n), pufage(n), nskins(n)
          endif
  310   continue
        outtim = outtim + delout
      endif
      do 311 n = 1, npufmx
        idump(n) = 0
  311 continue
c
c---------------------------------------------------------------------------
c
c  loop over plume sources and release new puffs if necessary 
c 
      if (idiag.ne.0) write(*,*)'beginning puff release loop'
      do 500 nn = 1, nplm 
        if ((curtim-tsrc(nn) .lt. delpuf).and.(tsrc(nn).gt.0.)) 
     &      go to 500 
        if (idiag.ne.0) write(*,*)'releasing new puff for plume # ',nn
        tsrc(nn) = curtim
c
c  find previous puff released from this source and set the
c  duration of puff dumping for that puff.
c
        tlast = -99.
        do 320 npff = 1,npufmx
          if (nskins(npff) .eq. 0) go to 320
          if (nplume(npff) .ne. npsrc(nn)) go to 320
          if (tpuff(npff) .gt. 0.) go to 320
          npsav = npff
          tlast = abs(tpuff(npsav))
          goto 321
  320   continue
  321   if (tlast.ne.-99.) then
          tpuff(npsav) = curtim - tlast
        endif
c
c   note that tpuff is initially the time at which the puff was
c   released.   once the next puff is released, however, it is
c   changed to the time elapsed between releases.
c   note - first half of check is to avoid dumping a puff too
c          long if its source is later combined with another
c
        npuffs = npuffs + 1
        kcount = kcount + 1
c
c     (note -- npuffs is the number of puffs currently
c     running.
c     kcount is the number of puffs released since
c     the start of the run, including those which
c     have already been dumped.)
c
c  find coarse grid location of new puff
c
        xc = xplm(nn)
        yc = yplm(nn)
        if (lcart) then
          jc = ifix(yc/deltay) + 1
          ic = ifix(xc/deltax(jc)) + 1
        else
          jc = ifix(yc) + 1
          ic = ifix(xc) + 1
        endif
c
c  is new puff outside coarse grid domain?
c
        if (istr(jc).gt.nox .or. ic.le.istr(jc) .or. 
     &      ic.ge.iend(jc)) then
          npuffs = npuffs - 1
          tsrc(nn) = -tsrc(nn)
          write(*,22340) ic, jc, nox, noy, xplm(nn), yplm(nn),
     &                       nn, xorg, yorg, deltax(jc), deltay
22340     format(1x,'*** source outside region ***',/,
     &             1x,'   ic, jc, nx, ny ',4i10,/,
     &             1x,'   xplm(nn), yplm(nn), nn ',2e12.4,i10,/,
     &             1x,'   xorg, yorg, deltax, deltay ',4e12.4)
          go to 500
        endif
c
c  loop over puffs to find an empty puff index to load
c 
        do 400 n = 1, npufmx
          if (nskins(n) .ne. 0 ) go to 400
          if (idiag.ne.0) write(*,*) 'new puff will be #:',n
          nsrcmbn(n) = 0
c
          pufage(n) = curtim
c
c  set height to plume height calculated by beh084
c

          height(n) = heff(npsrc(nn))/1000.
          do 410 k = 1,nlayer
            if (height(n).lt.h(ic,jc,k)) then
              kc = k
              goto 411
            endif
  410     continue
          kc = mxz + 100
          npuffs = npuffs - 1
          tsrc(nn) = -tsrc(nn)
          write(*,22341) ic, jc, kc, nox, noy, xplm(nn), yplm(nn),
     &                       n, xorg, yorg, deltax(jc), deltay
22341     format(1x,'*** source outside region ***',/,
     &           1x,'   ic, jc, kc, nx, ny ',5i10,/,
     &           1x,'   xplm(nn), yplm(nn), nn ',2e12.4,i10,/,
     &           1x,'   xorg, yorg, deltax, deltay ',4e12.4)
          goto 500
  411     continue
c
c  determine which fine grid (if any) puff is located in
c
          ifgnow = 0
          do 420 ifine=1,numfin
            if (ic .ge. ixfb(ifine) .and. ic .le. ixfe(ifine) .and.
     &             jc .ge. jyfb(ifine) .and. jc .le. jyfe(ifine)) then
              ifgnow = ifine
            endif
  420     continue
c
c  find fine grid locations
c
          if (ifgnow.gt.0) then
            if (lcart) then
              yf = yc-float(jyfb(ifgnow)-1)*deltay+delfy(ifgnow)
              jf = ifix(yf/delfy(ifgnow)) + 1
              xf = xc-float(ixfb(ifgnow)-1)*deltax(jc)+delfx(jf,ifgnow)
              if = ifix(xf/delfx(jf,ifgnow)) + 1
            else
              yf = (yc-float(jyfb(ifgnow)-1))*float(nhf(ifgnow))+1.
              jf = ifix(yf) + 1
              xf = (xc-float(ixfb(ifgnow)-1))*float(nhf(ifgnow))+1.
              if = ifix(xf) + 1
            endif
          endif
          if (idiag.ne.0.and.n.eq.1) then
            write(*,*) 'xc,yc,height,ic,jc,kc,ifgnow,if,jf,kf'
            write(*,*)  xc,yc,height(n),ic,jc,kc,ifgnow,if,jf,kf 
          endif
c
c  initialize variables for calculating trajectory step
c
          dtrem = deltat
          dttot = 0.
          dstot = 0.
c
c  trjstp takes a step that reaches the boundary of the finest grid cell 
c  (top, bottom or one of the sides) that the current puff occupies.  keep 
c  taking steps with trjstp until one coarse grid step is completed.
c
  430     ifgold = ifgnow
          if (ifgnow .eq. 0) then
            if (lcart) then 
              xxc = xc
              yyc = yc
            else
              xxc = xc*deltax(jc)
              yyc = yc*deltay
            endif
            call trjstp(mxx,mxy,mxz,nox,noy,nlayer,u,v,h,deltax(jc),
     &                     deltay,xxc,yyc,height(n),dtrem,dtc,dels,
     &                     ucomp,vcomp,ic,jc,kc,idiag)
            if (lcart) then 
              xc = xxc
              yc = yyc
            else
              xc = xxc/deltax(jc)
              yc = yyc/deltay
            endif
          else
            if (lcart) then 
              xxf = xf
              yyf = yf
            else
              xxf = xf*delfx(jf,ifgnow)
              yyf = yf*delfy(ifgnow)
            endif
            call trjstp(nfx(ifgnow),nfy(ifgnow),mxfz,nfx(ifgnow),
     &                     nfy(ifgnow),nfz(ifgnow),uf(1,1,ifgnow),
     &                     vf(1,1,ifgnow),htf(1,1,ifgnow),
     &                     delfx(jf,ifgnow),delfy(ifgnow),xxf,yyf,
     &                     height(n),dtrem,dtc,dels,ucomp,vcomp,
     &                     if,jf,kf,idiag)
            if (lcart) then 
              xf = xxf
              yf = yyf
            else
              xf = xxf/delfx(jf,ifgnow)
              yf = yyf/delfy(ifgnow)
            endif
c
c  find new coarse grid location
c
            if (lcart) then
              xc = xf-delfx(jf,ifgnow)
     &                +float(ixfb(ifgnow)-1)*deltax(jc)
              yc = yf-delfy(ifgnow)+float(jyfb(ifgnow)-1)*deltay
              jc = ifix(yc/deltay) + 1
              ic = ifix(xc/deltax(jc)) + 1
            else
              xc = (xf-1.)/float(nhf(ifgnow))
     &                +float(ixfb(ifgnow)-1)
              yc = (yf-1.)/float(nhf(ifgnow))+float(jyfb(ifgnow)-1)
              jc = ifix(yc) + 1
              ic = ifix(xc) + 1
            endif
            do 440 k = 1,nlayer
              if (height(n).lt.h(ic,jc,k)) then
                kc = k
                goto 441
              endif
  440       continue
            kc = mxz + 100
  441       continue
          endif
c
c  is puff outside coarse grid domain?
c
          if (istr(jc).gt.nox .or. ic.le.istr(jc) .or. ic.ge.iend(jc)
     &          .or. kc.gt.nlayer) then
            do 445 is = 1, nospec
              pxflow(is) = pxflow(is) + ptemis(is,npsrc(nn))*deltat
  445       continue
            npuffs = npuffs - 1
            tsrc(nn) = -tsrc(nn)
            write(*,22342) ic, jc, kc, nox, noy, xplm(nn), 
     &                    yplm(nn),nn, xorg, yorg, deltax(jc), deltay
22342       format(1x,'*** puff outside region in 1 timestep ***',/,
     &                1x,'   ic, jc, kc, nx, ny ',5i10,/,
     &                1x,'   xplm(nn), yplm(nn), nn ',2e12.4,i10,/,
     &                1x,'   xorg, yorg, deltax, deltay ',4e12.4)
            go to 500
          endif
c
c  determine which fine grid (if any) puff is located in
c
          ifgnow = 0
          do 450 ifine=1,numfin
            if (ic .ge. ixfb(ifine) .and. ic .le. ixfe(ifine) .and.
     &           jc .ge. jyfb(ifine) .and. jc .le. jyfe(ifine)) then
                 ifgnow = ifine
            endif
  450     continue
c
c  if puff is in a new fine grid, find new fine grid locations
c
          if (ifgnow.ne.ifgold.and.ifgnow.gt.0) then
            if (ifgnow.gt.ifgold) then
              write(*,*)'puff #',n,' entering a finer grid'
            endif
            if (lcart) then
              yf = yc-float(jyfb(ifgnow)-1)*deltay+delfy(ifgnow)
              jf = ifix(yf/delfy(ifgnow)) + 1
              xf = xc-float(ixfb(ifgnow)-1)*deltax(jc)+delfx(jf,ifgnow)
              if = ifix(xf/delfx(jf,ifgnow)) + 1
            else
              yf = (yc-float(jyfb(ifgnow)-1))*float(nhf(ifgnow))+1.
              jf = ifix(yf) + 1
              xf = (xc-float(ixfb(ifgnow)-1))*float(nhf(ifgnow))+1.
              if = ifix(xf) + 1
            endif
          endif
c
c   update new fine grid layer indices for puff
c
          if ( ifgnow .gt. 0 ) then
            call fkindx(nfx(ifgnow),nfy(ifgnow),mxfz,
     &                  nfx(ifgnow),nfy(ifgnow),nfz(ifgnow),
     &                  htf(1,1,ifgnow),height(n),
     &                  if,jf,kf,idiag)
          endif
c
c  update trajectory length, remaining time and total time
c  loop back to 430 to finish coarse grid timestep
c
          dstot = dstot + dels
          dttot = dttot + dtc
          dtrem = deltat - dttot
          if (dtrem .gt. .01) go to 430
          if (idiag.ne.0.and.n.eq.1) then
            write(*,*) 'n,xc,yc,height,ic,jc,kc,ifgnow,if,jf,kf'
            write(*,*)  n,xc,yc,height(n),ic,jc,kc,ifgnow,if,jf,kf 
          endif
c
c  load puff-dependent location variables
c  make tpuff < 0 to show that it is time of day, not duration
c
          tpuff(n) = -curtim
          nplume(n) = npsrc(nn)
          nskins(n) = -nrclls
          xloc(n) = xc
          yloc(n) = yc
          xlnpuf(n) = dstot
          wnd(n) = dstot/dttot
          dtpuf(n) = dttot
          kpuff(n) = kcount
          ifg(n) = ifgnow
          iic(n) = ic
          jjc(n) = jc
          kkc(n) = kc
          iif(n) = if
          jjf(n) = jf
          kkf(n) = kf
          if (idiag.ne.0.and.n.eq.1) then
            write(*,*)'n,tpuff,nplume,nskins,xloc,yloc,height',
     &                'xlnpuf,dtpuf,kpuff'
            write(*,*) n,tpuff(n),nplume(n),nskins(n),xloc(n),
     &                 yloc(n),height(n),xlnpuf(n),dtpuf(n),kpuff(n)
          endif
c
c  expand puff via stability class
c
          if (ifgnow .eq. 0) then
            call expand(mxx,mxy,mxz,nlayer,ic,jc,kc,u,v,temper,
     &                   tsurf(ic,jc),press,h,kv,height(n),sigma(n),
     &                   sigmav(n),rfacb(nrclls+1),xlnpuf(n),dstot,
     &                   wdump,deltax(jc),deltay,idump(n),1,idiag,
     &                   stkd(nplume(n)),stkht(nplume(n)),
     &                   heff(nplume(n)),n)
          else
            call expandf(nfx(ifgnow),nfy(ifgnow),mxfz,nfz(ifgnow),
     &                     if,jf,kf,uf(1,1,ifgnow),vf(1,1,ifgnow),
     &                     temprf(1,1,ifgnow),tsurf(ic,jc),
     &                     presf(1,1,ifgnow),htf(1,1,ifgnow),
     &                     fkv(1,1,ifgnow),height(n),sigma(n),
     &                     sigmav(n),rfacb(nrclls+1),xlnpuf(n),dstot,
     &                     wdump,delfx(jf,ifgnow),delfy(ifgnow),
     &                     idump(n),1,idiag,stkd(nplume(n)),
     &                     stkht(nplume(n)),heff(nplume(n)),n)
            do 461 k = 1,nlayer
              if (height(n).lt.h(ic,jc,k)) then
                kc = k
                kkc(n) = kc
                goto 462
              endif
  461       continue
  462       continue
          endif
c
c  load puff emissions for rpm
c
          if (idiag.ne.0.and.n.eq.1) write(*,*) 
     &               'plume #, source #, puff #, species #, emrate'
          do 470 isp = 1,nemit
            is = kemit(isp)
            if (is .le. 0) go to 470
            emrate(is+1,n) = ptemis(isp,npsrc(nn))
            if (idiag.ne.0.and.n.eq.1) 
     &        write(*,88) nn, npsrc(nn), n, is,emrate(is+1,n)
  470     continue
   88     format('nn, npsrc(nn), n, is, emrate(is+1,n): ',4i5,e11.4)
c
c
c  Set up some checks on puff dimensions ...... GEM
c  The following checks have been implemented due to some problems
c  encountered with P-i-G sources initialized in layers with calm winds.
c  These puffs end up with such small dimensions that the resulting
c  concentrations in the plumes are unrealistic and cause the chemistry
c  to blow up.  The 100X dilution is set based on estimates of the 
c  dilution from the stack for a typical 6 min. time step. 
c 
c  The NOx concentrations are also checked against a max of 5ppm. This
c  is based on a stack concentration of 500ppm (which is somewhat large)
c  but might be possible.
c
c  Check for 100X dilution ... If not, increase length to plume rise
c  check again and if still not 100X dilution, increase all dimensions
c  proportionally.
c
           volstk=pi*stkd(nplume(n))**2*vexit(nplume(n))*dtpuf(n)/4.0
	   volpuff=4.0*pi*sigma(n)*sigmav(n)*xlnpuf(n)*1000.
	   if ( 100.*volstk .gt. volpuff ) then
	     icnt=icnt+1
c
            if ( idiag .ne. 0 ) then
             write(*,*) '*** Dilution < 100X ... increasing xlnpuf'
	     write(*,*) 'n,nplume(n) :'
	     write(*,*) n,nplume(n)
	     write(*,*) 'Old Values: sigma,sigmav,xlnpuf,wdump,idump :'
	     write(*,*) sigma(n),sigmav(n),xlnpuf(n),wdump,idump(n) 
	     write(*,*) 'stkd,stkht,heff :'
	     write(*,*) stkd(nplume(n)),stkht(nplume(n)),heff(nplume(n))
	     write(*,*) 'vexit,dtpuf :'
	     write(*,*) vexit(nplume(n)),dtpuf(n)
	     write(*,*) 'volstk,volpuff :'
	     write(*,*) volstk,volpuff
	    endif
c
	     xlnpuf(n)=(heff(nplume(n))-stkht(nplume(n)))/1000.
	     volpuff=4.0*pi*sigma(n)*sigmav(n)*xlnpuf(n)*1000.
c
            if ( idiag .ne. 0 ) then
	     write(*,*) 'New Values: xlnpuf, volpuff :'
	     write(*,*) xlnpuf(n),volpuff
	    endif
c
	     if ( 100.*volstk .gt. volpuff ) then
	       delv=(100.*volstk/volpuff)**(1./3.)
	       sigma(n)=sigma(n)*delv
       	       sigmav(n)=sigmav(n)*delv
       	       xlnpuf(n)=xlnpuf(n)*delv
	       volpuff=4.0*pi*sigma(n)*sigmav(n)*xlnpuf(n)*1000.
c
             if ( idiag .ne. 0 ) then
             write(*,*) '*** Dilution still < 100X  ... ',
     &                    ' increasing all dimesnions'
 	     write(*,*) 'New Values: sigma,sigmav,xlnpuf,volpuff,delv :'
             write(*,*) sigma(n),sigmav(n),xlnpuf(n),volpuff,delv
	     endif
c
	     endif
	   endif
c
c  Now check the NOx concentrations in the puff  must <= 5ppm
c
          cnoxe=(emrate(kno+1,n)+emrate(kno2+1,n))*dtpuf(n)/nrclls
          cnox=cnoxe/(pi*rfacb(2)**2*sigma(n)*sigmav(n)*xlnpuf(n)*1000.)
           if ( cnox .gt. 5.0/24300. ) then
	     if ( idiag .ne. 0 ) then
              write(*,*) ' NOx Concentrations > 5 ppm ...',
     &                    ' increasing all dimensions'
	      write(*,*) 'n,nplume(n) :'
	      write(*,*) n,nplume(n)
 	      write(*,*) 'Old Values: cnox,sigma,sigmav,xlnpuf :'
	      write(*,*) cnox*24300.,sigma(n),sigmav(n),xlnpuf(n)
	     endif
c
	      icnt=icnt+1
	      delv=(cnox/(5./24300.))**(1./3.)
	      sigma(n)=sigma(n)*delv
       	      sigmav(n)=sigmav(n)*delv
       	      xlnpuf(n)=xlnpuf(n)*delv
c
          cnox=cnoxe/(pi*rfacb(2)**2*sigma(n)*sigmav(n)*xlnpuf(n)*1000.)
	    if ( idiag .ne. 0 ) then
 	      write(*,*) 'New Values: cnox,sigma,sigmav,xlnpuf,delv :'
	      write(*,*) cnox*24300.,sigma(n),sigmav(n),xlnpuf(n),delv
	    endif
c
	   endif
           wnd(n) = xlnpuf(n)/dttot
c
c
c   set up some parameters for rpm chemistry
c
          if (ifgnow.eq.0) then
            call setprm(mxx,mxy,mxz,mxspec,n,ic,jc,kc,nospec,
     &                  temper,conc,deltat,deltax(jc),deltay)
          else
            call setprm(nfx(ifgnow),nfy(ifgnow),nfz(ifgnow),nospec,
     &                n,if,jf,kf,nospec,
     &                temprf(1,1,ifgnow),cncf(1,1,1,ifgnow),
     &                deltat,delfx(jf,ifgnow),delfy(ifgnow))
          endif
c
c  this section is for combining puffs
c
          do 480 kcom = 1,npufmx
            if (nskins(kcom) .ge. 0) go to 480
            if (kcom .eq. n) go to 480
c
c  check x and y coordinates
c
            if (lcart) then
              dsq = (xloc(n)-xloc(kcom))**2+(yloc(n)-yloc(kcom))**2
            else
              dsq = (dlong*(xloc(n)-xloc(kcom)))**2
     &              +(dlat*(yloc(n)-yloc(kcom)))**2
            endif
            if (dsq .gt. xcmbn**2) go to 480
c
c  check z coordinates
c
            kc1 = 0
            kc2 = 0
            do 481 k = 1,nlayer
              if (height(n).lt.h(ic,jc,k).and.kc1.eq.0) then
                kc1 = k
              endif
              if (height(kcom).lt.h(ic,jc,k).and.kc2.eq.0) then
                kc2 = k
              endif
  481       continue
            if (kc2 .ne. kc1) go to 480
c
c  found puffs close enough to combine
c
            nsrcmbn(kcom) = nsrcmbn(kcom) + 1
            isrcmbn(kcom,nsrcmbn(kcom)) = nplume(n)
            if (idiag.ne.0.and.(n.eq.1.or.kcom.eq.1)) then
              write(*,*) 'combining puffs',kcom,n
              write(*,*) 'from sources',nplume(kcom),nplume(n)
              write(*,*) 'this puff is a combination of puffs from',
     &                        nsrcmbn(kcom)+1,' sources'
            endif
            xloc(kcom) = (xloc(kcom) + xloc(n))*.5
            yloc(kcom) = (yloc(kcom) + yloc(n))*.5
            height(kcom) = (height(kcom) + height(n))*.5
            do 482 isp = 1,nemit
              is = kemit(isp)
              if (is .le. 0) go to 482
              emrate(is+1,kcom) = emrate(is+1,kcom) + emrate(is+1,n)
  482       continue
            npuffs = npuffs - 1
            kcount = kcount - 1
            nskins(n) = 0
            go to 500
  480     continue
          go to 500
  400   continue
        write(nutrc,370) npufmx
        write(*,370) npufmx
        write(ierrlg,371) 1121,npufmx
        stop
  500 continue
c
c   call rpm
c
      if (idiag.ne.0) write(*,*) '3rd rpm call'
      call rpm(nutrc,nospec,deltat,time,idate,2,lerror)
      if(lerror) go to 800
c
c   call rpm zero emissions puffs
c
      if (idiag.ne.0) write(*,*) '4th rpm call'
      call rpm(nutrc,nospec,deltat,time,idate,3,lerror)
      if(lerror) go to 800
c
c  all puffs are current: update current time
c
      curtim = curtim + deltat 
c
c  determine if a portion (or all) of the puff should be dumped into the grid
c
      do 600 n = 1,npufmx
        if (nskins(n).eq.0) goto 600
        nskins(n) = abs(nskins(n))
        if (ifg(n).eq.0) then
          call puffdmp(mxx,mxy,mxz,n,ifg(n),iic(n),jjc(n),kkc(n),
     &              iic(n),jjc(n),kkc(n),h,deltay,deltat,idiag)
        else
          call puffdmp(nfx(ifg(n)),nfy(ifg(n)),mxfz,n,ifg(n),
     &              iic(n),jjc(n),kkc(n),iif(n),jjf(n),kkf(n),
     &              htf(1,1,ifg(n)),delfy(ifg(n)),deltat,idiag)
        endif
  600 continue
c
       do 3111 n = 1, npufmx
        idump(n) = 0
 3111  continue
c
c  Temporary diagnostics: write out all species concentrations for all 
c  puff skins for puff number 1
c
      if (idiag.ne.0.and.mod(nint(time),100).eq.0) then
         write(*,*)
         write(*,*)
         write(*,*)'Concentrations in puff #1: this puff has',
     &             abs(nskins(1)),' skins'
         write(*,750)
         do 700 is = 1,nospec+1
            do 701 ib = 1,nrclls
               if (ib.le.abs(nskins(1))) then
                  out1(ib) = concs(is,ib,1)
                  out2(ib) = connul(is,ib,1)
                  arfac = pi*sigma(1)*sigmav(1)*
     &                    (rfacb(ib+1)**2 - rfacb(ib)**2)
                  out3(ib) = arfac*out1(ib)
                  out4(ib) = arfac*out2(ib)
               else
                  out1(ib) = 0.
                  out2(ib) = 0.
                  out3(ib) = 0.
                  out4(ib) = 0.
               endif
 701        continue
            if (is.eq.1) then
               write(*,*)
               write(*,760) is,(out1(ib),ib=1,nrclls),
     &                   (out2(ib),ib=1,nrclls)
               write(*,*)
               write(*,762) is,(out3(ib),ib=1,nrclls),
     &                   (out4(ib),ib=1,nrclls)
            else
               write(*,*)
               write(*,761) is,(mspec(i,is-1),i=1,4),
     &                   (out1(ib),ib=1,nrclls),(out2(ib),ib=1,nrclls)
               write(*,*)
               write(*,763) is,(mspec(i,is-1),i=1,4),
     &                   (out3(ib),ib=1,nrclls),(out4(ib),ib=1,nrclls)
            endif
 700     continue
         write(*,*)
         write(*,*)
      endif
 750  format('                  CONCS and MASS in Shell #',
     &       '      |        CONNUL in Shell #',/,
     &       ' Species     1        2        3        4   |',
     &       '    1       2       3       4',/,
     &       '--------------------------------------------|',
     &       '--------------------------------')
 760  format(i2,'  TRAC',8(1pe9.1))
 761  format(i2,2x,4a1,8(1pe9.1))
 762  format(i2,'  TRAC',8(1pe12.4))
 763  format(i2,2x,4a1,8(1pe12.4))
c
c                      CONCS in Shell #      |        CONNUL in Shell #
c Species     1        2        3        4   |    1       2       3       4
c--------------------------------------------|--------------------------------
c 1  MGLY  e.eE+ee  e.eE+ee  e.eE+ee  e.eE+ee  e.eE+ee e.eE+ee e.eE+ee e.eE+ee
c
c
      if (idiag.ne.0) write(*,*) 'leaving ttraj'
c     call flush(9)
      return
c
c   --- error in rpm ---
c
  800 write(nutrc,2800) (ipf,kpuff(ipf),nplume(ipf),ipf=1,mxpfs)
      write(*,2800) (ipf,kpuff(ipf),nplume(ipf),ipf=1,mxpfs)
      write(ierrlg,2801) 1157,(ipf,kpuff(ipf),nplume(ipf),ipf=1,mxpfs)
 2800 format('rpm has reported an error',//,
     &       1x,'puff',2x,'count',1x,'source',/,
     &       (1x,i5,1x,i5,1x,i5))
 2801 format(i4,1x,'rpm has reported an error',//,
     &       1x,'puff',2x,'count',1x,'source',/,
     &       (1x,i5,1x,i5,1x,i5))
      stop
c
c     --- formats ---
c
  350 format (////, 5x, ' pf#', 1x,'count',1x,'sourc',
     &        2x,'x-y coordinates',2x,'height',2x,'sigma',3x,'sigmav',
     &        1x,'dmp',1x,'releas',3x,'delt',3x,' t.o.b. ','skins')
  351 format (/,10x,i4,
     &  ' puffs are currently running.  current time is ',
     &  f8.2,' minutes.')
  360 format (5x,i4,1x,i5,1x,i5,1x,f8.2,1x,f8.2,1x,f6.2,
     &        1x,f6.0,1x,f6.0,1x,i3,1x,f8.2,1x,f8.3,1x,f8.2,1x,i3)
  370 format( ///, ' maximum number of puffs, ', i5, ', exceeded')
  371 format(i4,1x,///,' maximum number of puffs, ', i5,
     &        ', exceeded')
      end
